# encoding: utf-8
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models

class Migration(SchemaMigration):

    def forwards(self, orm):
        
        # Adding model 'PublicEnrollment'
        db.create_table('tx_highered_publicenrollment', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('year', self.gf('django.db.models.fields.IntegerField')(default=1970)),
            ('year_type', self.gf('django.db.models.fields.CharField')(max_length=10, null=True)),
            ('institution', self.gf('django.db.models.fields.related.ForeignKey')(related_name='publicenrollment', to=orm['tx_highered.Institution'])),
            ('total', self.gf('django.db.models.fields.IntegerField')(null=True)),
            ('african_american_count', self.gf('django.db.models.fields.IntegerField')(null=True)),
            ('asian_count', self.gf('django.db.models.fields.IntegerField')(null=True)),
            ('hispanic_count', self.gf('django.db.models.fields.IntegerField')(null=True)),
            ('international_count', self.gf('django.db.models.fields.IntegerField')(null=True)),
            ('multiracial_count', self.gf('django.db.models.fields.IntegerField')(null=True)),
            ('native_american_count', self.gf('django.db.models.fields.IntegerField')(null=True)),
            ('pacific_islander_count', self.gf('django.db.models.fields.IntegerField')(null=True)),
            ('unknown_count', self.gf('django.db.models.fields.IntegerField')(null=True)),
            ('white_count', self.gf('django.db.models.fields.IntegerField')(null=True)),
            ('african_american_percent', self.gf('django.db.models.fields.FloatField')(null=True)),
            ('asian_percent', self.gf('django.db.models.fields.FloatField')(null=True)),
            ('hispanic_percent', self.gf('django.db.models.fields.FloatField')(null=True)),
            ('international_percent', self.gf('django.db.models.fields.FloatField')(null=True)),
            ('multiracial_percent', self.gf('django.db.models.fields.FloatField')(null=True)),
            ('native_american_percent', self.gf('django.db.models.fields.FloatField')(null=True)),
            ('pacific_islander_percent', self.gf('django.db.models.fields.FloatField')(null=True)),
            ('unknown_percent', self.gf('django.db.models.fields.FloatField')(null=True)),
            ('white_percent', self.gf('django.db.models.fields.FloatField')(null=True)),
        ))
        db.send_create_signal('tx_highered', ['PublicEnrollment'])

        # Adding unique constraint on 'PublicEnrollment', fields ['year', 'institution']
        db.create_unique('tx_highered_publicenrollment', ['year', 'institution_id'])


    def backwards(self, orm):
        
        # Removing unique constraint on 'PublicEnrollment', fields ['year', 'institution']
        db.delete_unique('tx_highered_publicenrollment', ['year', 'institution_id'])

        # Deleting model 'PublicEnrollment'
        db.delete_table('tx_highered_publicenrollment')


    models = {
        'tx_highered.admissions': {
            'Meta': {'ordering': "['year']", 'unique_together': "(('year', 'institution'),)", 'object_name': 'Admissions'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'institution': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'admissions'", 'to': "orm['tx_highered.Institution']"}),
            'number_admitted': ('django.db.models.fields.IntegerField', [], {'null': 'True', 'blank': 'True'}),
            'number_admitted_who_enrolled': ('django.db.models.fields.IntegerField', [], {'null': 'True', 'blank': 'True'}),
            'number_of_applicants': ('django.db.models.fields.IntegerField', [], {'null': 'True', 'blank': 'True'}),
            'percent_of_admitted_who_enrolled': ('django.db.models.fields.DecimalField', [], {'null': 'True', 'max_digits': '4', 'decimal_places': '1', 'blank': 'True'}),
            'percent_of_applicants_admitted': ('django.db.models.fields.DecimalField', [], {'null': 'True', 'max_digits': '4', 'decimal_places': '1', 'blank': 'True'}),
            'percent_top10rule': ('django.db.models.fields.DecimalField', [], {'null': 'True', 'max_digits': '4', 'decimal_places': '1'}),
            'year': ('django.db.models.fields.IntegerField', [], {'default': '1970'}),
            'year_type': ('django.db.models.fields.CharField', [], {'max_length': '10', 'null': 'True'})
        },
        'tx_highered.enrollment': {
            'Meta': {'ordering': "['year']", 'unique_together': "(('year', 'institution'),)", 'object_name': 'Enrollment'},
            'fulltime': ('django.db.models.fields.IntegerField', [], {'null': 'True'}),
            'fulltime_equivalent': ('django.db.models.fields.IntegerField', [], {'null': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'institution': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'enrollment'", 'to': "orm['tx_highered.Institution']"}),
            'parttime': ('django.db.models.fields.IntegerField', [], {'null': 'True'}),
            'total': ('django.db.models.fields.IntegerField', [], {'null': 'True'}),
            'total_percent_asian': ('django.db.models.fields.IntegerField', [], {'null': 'True'}),
            'total_percent_black': ('django.db.models.fields.IntegerField', [], {'null': 'True'}),
            'total_percent_hispanic': ('django.db.models.fields.IntegerField', [], {'null': 'True'}),
            'total_percent_native': ('django.db.models.fields.IntegerField', [], {'null': 'True'}),
            'total_percent_unknown': ('django.db.models.fields.IntegerField', [], {'null': 'True'}),
            'total_percent_white': ('django.db.models.fields.IntegerField', [], {'null': 'True'}),
            'year': ('django.db.models.fields.IntegerField', [], {'default': '1970'}),
            'year_type': ('django.db.models.fields.CharField', [], {'max_length': '10', 'null': 'True'})
        },
        'tx_highered.graduationrates': {
            'Meta': {'ordering': "['year']", 'unique_together': "(('year', 'institution'),)", 'object_name': 'GraduationRates'},
            'bachelor_4yr': ('django.db.models.fields.IntegerField', [], {'null': 'True'}),
            'bachelor_5yr': ('django.db.models.fields.IntegerField', [], {'null': 'True'}),
            'bachelor_6yr': ('django.db.models.fields.IntegerField', [], {'null': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'institution': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'graduationrates'", 'to': "orm['tx_highered.Institution']"}),
            'year': ('django.db.models.fields.IntegerField', [], {'default': '1970'}),
            'year_type': ('django.db.models.fields.CharField', [], {'max_length': '10', 'null': 'True'})
        },
        'tx_highered.institution': {
            'Meta': {'object_name': 'Institution'},
            'address': ('django.db.models.fields.CharField', [], {'max_length': '200', 'null': 'True', 'blank': 'True'}),
            'city': ('django.db.models.fields.CharField', [], {'max_length': '50', 'null': 'True', 'blank': 'True'}),
            'description': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'fice_id': ('django.db.models.fields.IntegerField', [], {'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'institution_type': ('django.db.models.fields.CharField', [], {'max_length': '30', 'null': 'True', 'blank': 'True'}),
            'ipeds_id': ('django.db.models.fields.IntegerField', [], {'null': 'True', 'blank': 'True'}),
            'is_private': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'location': ('django.contrib.gis.db.models.fields.PointField', [], {'null': 'True', 'blank': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '60'}),
            'ope_id': ('django.db.models.fields.IntegerField', [], {'null': 'True', 'blank': 'True'}),
            'phone': ('django.db.models.fields.CharField', [], {'max_length': '15', 'null': 'True', 'blank': 'True'}),
            'slug': ('django.db.models.fields.SlugField', [], {'unique': 'True', 'max_length': '60', 'db_index': 'True'}),
            'system': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['tx_highered.System']", 'null': 'True', 'blank': 'True'}),
            'url': ('django.db.models.fields.URLField', [], {'max_length': '200', 'null': 'True', 'blank': 'True'}),
            'wikipedia_abstract': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'wikipedia_logo': ('django.db.models.fields.files.ImageField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'wikipedia_scraped': ('django.db.models.fields.DateTimeField', [], {'null': 'True', 'blank': 'True'}),
            'wikipedia_seal': ('django.db.models.fields.files.ImageField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'wikipedia_title': ('django.db.models.fields.CharField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'zip_code': ('django.db.models.fields.CharField', [], {'max_length': '10', 'null': 'True', 'blank': 'True'})
        },
        'tx_highered.pricetrends': {
            'Meta': {'ordering': "['year']", 'unique_together': "(('year', 'institution'),)", 'object_name': 'PriceTrends'},
            'books_and_supplies': ('django.db.models.fields.IntegerField', [], {'null': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'institution': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'pricetrends'", 'to': "orm['tx_highered.Institution']"}),
            'room_and_board_off_campus': ('django.db.models.fields.IntegerField', [], {'null': 'True'}),
            'room_and_board_off_campus_w_family': ('django.db.models.fields.IntegerField', [], {'null': 'True'}),
            'room_and_board_on_campus': ('django.db.models.fields.IntegerField', [], {'null': 'True'}),
            'tuition_fees_in_state': ('django.db.models.fields.IntegerField', [], {'null': 'True'}),
            'tuition_fees_outof_state': ('django.db.models.fields.IntegerField', [], {'null': 'True'}),
            'year': ('django.db.models.fields.IntegerField', [], {'default': '1970'}),
            'year_type': ('django.db.models.fields.CharField', [], {'max_length': '10', 'null': 'True'})
        },
        'tx_highered.publicenrollment': {
            'Meta': {'ordering': "['year']", 'unique_together': "(('year', 'institution'),)", 'object_name': 'PublicEnrollment'},
            'african_american_count': ('django.db.models.fields.IntegerField', [], {'null': 'True'}),
            'african_american_percent': ('django.db.models.fields.FloatField', [], {'null': 'True'}),
            'asian_count': ('django.db.models.fields.IntegerField', [], {'null': 'True'}),
            'asian_percent': ('django.db.models.fields.FloatField', [], {'null': 'True'}),
            'hispanic_count': ('django.db.models.fields.IntegerField', [], {'null': 'True'}),
            'hispanic_percent': ('django.db.models.fields.FloatField', [], {'null': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'institution': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'publicenrollment'", 'to': "orm['tx_highered.Institution']"}),
            'international_count': ('django.db.models.fields.IntegerField', [], {'null': 'True'}),
            'international_percent': ('django.db.models.fields.FloatField', [], {'null': 'True'}),
            'multiracial_count': ('django.db.models.fields.IntegerField', [], {'null': 'True'}),
            'multiracial_percent': ('django.db.models.fields.FloatField', [], {'null': 'True'}),
            'native_american_count': ('django.db.models.fields.IntegerField', [], {'null': 'True'}),
            'native_american_percent': ('django.db.models.fields.FloatField', [], {'null': 'True'}),
            'pacific_islander_count': ('django.db.models.fields.IntegerField', [], {'null': 'True'}),
            'pacific_islander_percent': ('django.db.models.fields.FloatField', [], {'null': 'True'}),
            'total': ('django.db.models.fields.IntegerField', [], {'null': 'True'}),
            'unknown_count': ('django.db.models.fields.IntegerField', [], {'null': 'True'}),
            'unknown_percent': ('django.db.models.fields.FloatField', [], {'null': 'True'}),
            'white_count': ('django.db.models.fields.IntegerField', [], {'null': 'True'}),
            'white_percent': ('django.db.models.fields.FloatField', [], {'null': 'True'}),
            'year': ('django.db.models.fields.IntegerField', [], {'default': '1970'}),
            'year_type': ('django.db.models.fields.CharField', [], {'max_length': '10', 'null': 'True'})
        },
        'tx_highered.system': {
            'Meta': {'object_name': 'System'},
            'address': ('django.db.models.fields.CharField', [], {'max_length': '200', 'null': 'True', 'blank': 'True'}),
            'city': ('django.db.models.fields.CharField', [], {'max_length': '50', 'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'location': ('django.contrib.gis.db.models.fields.PointField', [], {'null': 'True', 'blank': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '60'}),
            'phone': ('django.db.models.fields.CharField', [], {'max_length': '15', 'null': 'True', 'blank': 'True'}),
            'slug': ('django.db.models.fields.SlugField', [], {'unique': 'True', 'max_length': '60', 'db_index': 'True'}),
            'url': ('django.db.models.fields.URLField', [], {'max_length': '200', 'null': 'True', 'blank': 'True'}),
            'zip_code': ('django.db.models.fields.CharField', [], {'max_length': '10', 'null': 'True', 'blank': 'True'})
        },
        'tx_highered.testscores': {
            'Meta': {'ordering': "['year']", 'unique_together': "(('year', 'institution'),)", 'object_name': 'TestScores'},
            'act_composite_25th_percentile': ('django.db.models.fields.IntegerField', [], {'null': 'True'}),
            'act_composite_75th_percentile': ('django.db.models.fields.IntegerField', [], {'null': 'True'}),
            'act_english_25th_percentile': ('django.db.models.fields.IntegerField', [], {'null': 'True'}),
            'act_english_75th_percentile': ('django.db.models.fields.IntegerField', [], {'null': 'True'}),
            'act_math_25th_percentile': ('django.db.models.fields.IntegerField', [], {'null': 'True'}),
            'act_math_75th_percentile': ('django.db.models.fields.IntegerField', [], {'null': 'True'}),
            'act_submitted_number': ('django.db.models.fields.IntegerField', [], {'null': 'True'}),
            'act_submitted_percent': ('django.db.models.fields.IntegerField', [], {'null': 'True'}),
            'act_writing_25th_percentile': ('django.db.models.fields.IntegerField', [], {'null': 'True'}),
            'act_writing_75th_percentile': ('django.db.models.fields.IntegerField', [], {'null': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'institution': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'testscores'", 'to': "orm['tx_highered.Institution']"}),
            'sat_math_25th_percentile': ('django.db.models.fields.IntegerField', [], {'null': 'True'}),
            'sat_math_75th_percentile': ('django.db.models.fields.IntegerField', [], {'null': 'True'}),
            'sat_submitted_number': ('django.db.models.fields.IntegerField', [], {'null': 'True'}),
            'sat_submitted_percent': ('django.db.models.fields.IntegerField', [], {'null': 'True'}),
            'sat_verbal_25th_percentile': ('django.db.models.fields.IntegerField', [], {'null': 'True'}),
            'sat_verbal_75th_percentile': ('django.db.models.fields.IntegerField', [], {'null': 'True'}),
            'sat_writing_25th_percentile': ('django.db.models.fields.IntegerField', [], {'null': 'True'}),
            'sat_writing_75th_percentile': ('django.db.models.fields.IntegerField', [], {'null': 'True'}),
            'year': ('django.db.models.fields.IntegerField', [], {'default': '1970'}),
            'year_type': ('django.db.models.fields.CharField', [], {'max_length': '10', 'null': 'True'})
        }
    }

    complete_apps = ['tx_highered']



from difflib import get_close_matches

from tx_highered.models import Institution


class FuzzyMatcher(object):
    def __init__(self, qs, field):
        self.lookups = {}
        for o in qs:
            raw_key = getattr(o, field)
            key = self.clean_key(raw_key)
            if key in self.lookups:
                raise ValueError(u"Duplicate key '%s'" % key)
            self.lookups[key] = o

    def clean_key(self, key):
        return key

    def match(self, key):
        key = self.clean_key(key)

        # Return the exact match if it exists
        if key in self.lookups:
            return self.lookups[key]
        # Fall back to the closest matching key
        fuzzy_matches = get_close_matches(key, self.lookups.keys())
        closest_key = fuzzy_matches[0]
        return self.lookups[closest_key]


class InstitutionFuzzyMatcher(FuzzyMatcher):
    def __init__(self):
        qs = Institution.objects.only('id', 'name')
        super(InstitutionFuzzyMatcher, self).__init__(qs, 'name')

    def clean_key(self, key):
        return key.lower()


def create_or_update(objects, **kwargs):
    defaults = kwargs.pop('defaults', {})
    row_count = objects.filter(**kwargs).update(**defaults)
    if row_count:
        return (None, row_count)
    else:
        kwargs.update(defaults)
        obj = objects.create(**kwargs)
        return (obj, 1)



"""
   Copyright 2014 The Texas Tribune

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.

   https://github.com/texastribune/ipeds_reporter/blob/master/ipeds_reporter/utils.py
"""
from collections import defaultdict, namedtuple
import csv
import re


class IpedsCSVReader(object):
    """
    A special CSV reader for IPEDS reports.
    """
    header = None
    header_parsed = None

    def __init__(self, fh, **kwargs):
        for key, value in kwargs.items():
            if hasattr(self, key):
                setattr(self, key, value)
        self._reader = self.get_reader(fh)
        self.parse_header()

    def get_reader(self, fh):
        return csv.reader(fh)

    def parse_header(self):
        """
        Extract information about what variables are in the CSV.
        """
        header = self._reader.next()
        self.header = header
        header_parsed = [None for __ in header]
        IPEDSCell = namedtuple('IPEDSCell', 'short_name, year, raw_code')
        # the first two columns are the institution id and name
        for idx, cell in enumerate(header):
            if idx < 2:
                header_parsed[idx] = cell
                continue
            # DELETEME this is just a hack to get around the last empty col
            if not cell:
                continue
            short_name, raw_code = re.match(r'(\w+)\s\((\w+)\)', cell).groups()
            # XXX begin copy pasted from ipeds_reporter/models.py
            butts = re.match(r'(DRV)?([a-zA-Z]+)(\d{4})', raw_code).groups()
            __, code, year = butts
            if int(year[:2]) == int(year[2:]) - 1:
                # I've only found post Y2K years this has happened
                year = u'20' + year[:2]
            # XXX end
            header_parsed[idx] = IPEDSCell(short_name, year, raw_code)
        self.header_parsed = header_parsed

    def __iter__(self):
        """
        Do stuff.
        """
        assert self.header_parsed is not None
        for row in self._reader:
            yield zip(self.header_parsed, row)

    def explain_header(self):
        """
        Explain what header information was extracted.
        """
        cells = [x for x in self.header_parsed if isinstance(x, tuple)]
        short_names = [x.short_name for x in cells]
        unique_short_name = list(set(short_names))
        years = defaultdict(list)
        for cell in cells:
            years[cell.short_name].append(cell.year)
        return {
            'columns': len(cells),  # shape
            'unique short_names': unique_short_name,
            'years': years,
        }



import logging
from collections import defaultdict
from urllib2 import URLError

from django.contrib.gis import geos
from django.contrib.gis.db import models
from django.core.exceptions import ObjectDoesNotExist
from django.core.urlresolvers import reverse
from django.db.models import Q
from django.template import Context, TemplateDoesNotExist
from django.template.defaultfilters import slugify
from django.template.loader import get_template
from django.utils.functional import cached_property

try:
    from geopy import geocoders
    CAN_GEOCODE = True
except ImportError:
    CAN_GEOCODE = False

from tx_lege_districts.models import District
from tx_lege_districts.constants import SBOE


class SummarySentences(object):
    def __init__(self, obj):
        self.obj = obj
        self.path = '%s/sentences/%s/%%s.txt' % (obj._meta.app_label,
                obj._meta.module_name)
        self.cache = {}

    def __getattr__(self, key):
        if not key in self.cache:
            self.cache[key] = self.generate_sentence(key)
        return self.cache[key]

    def generate_sentence(self, name):
        try:
            t = get_template(self.path % name)
            c = Context({'obj': self.obj})
            return t.render(c).strip()
        except TemplateDoesNotExist:
            return None


class DistrictsSentence(object):
    def __init__(self, obj, *districts):
        self.obj = obj
        self.districts = districts

    def __unicode__(self):
        clauses = map(self.make_clause, self.districts)
        if len(clauses) <= 2:
            joined_clauses = u' and '.join(clauses)
        else:
            clauses[-1] = u'and %s' % (clauses[-1])
            joined_clauses = ', '.join(clauses)

        return u'%s is represented by %s' % (
            self.obj.sentence_name, joined_clauses)

    def make_clause(self, district):
        representative = district.representative
        if hasattr(district, 'get_absolute_url'):
            district_string = u'<a href="%s">%s</a>' % (
                district.get_absolute_url(), district)
        else:
            district_string = unicode(district)

        if not representative:
            return district_string
        elif hasattr(representative, 'get_absolute_url'):
            representative_string = u'<a href="%s">%s</a>' % (
                representative.get_absolute_url(), representative)
        else:
            representative_string = unicode(representative)

        return u'%s in %s' % (representative_string, district_string)


APP_LABEL = 'tx_highered'


INSTITUTION_CHOICES = (
        ("uni", "University"),
        ("med", "Health-Related Institutions"),
        ("pub_cc", "Community College"),
        ("pub_tech", "Technical College System"),
        ("pub_state", "State Colleges"),
        ("pri_jr", "Junior College"),
        ("pri_chi", "Chiropractic Institution"),
    )


class ContactFieldsMixin(models.Model):
    address = models.CharField(max_length=200, null=True, blank=True)
    city = models.CharField(max_length=50, null=True, blank=True)
    zip_code = models.CharField(max_length=10, null=True, blank=True)
    phone = models.CharField(max_length=15, null=True, blank=True)
    url = models.URLField(null=True, blank=True)
    location = models.PointField(geography=True, null=True, blank=True)

    objects = models.GeoManager()

    class Meta:
        abstract = True
        app_label = APP_LABEL

    def _guess_location(self, address_array):
        if not CAN_GEOCODE:
            return
        g = geocoders.Google()
        address = ", ".join(address_array)
        _, latlng = g.geocode(address)
        self.location = geos.fromstr("POINT({1} {0})".format(*latlng))
        self.save()
        return self.location

    def guess_location(self):
        logger = logging.getLogger('tx_highered.models.geolocate')
        if not CAN_GEOCODE:
            logger.error(u'Attempted to geocode without geopy installed')
            return

        # TODO better logging messages
        try:
            guess1 = [self.address, self.city, self.zip_code]
            self._guess_location(guess1)
            logger.debug(self.location)
        except (ValueError, URLError, geocoders.google.GQueryError) as e:
            logger.error("%s %s" % (e.message, ", ".join(guess1)))
            # try again
            try:
                guess2 = [self.name, self.zip_code]
                self._guess_location(guess2)
                logger.debug(self.location)
            except (ValueError, URLError, geocoders.google.GQueryError) as e:
                logger.error("%s %s" % (e.message, ", ".join(guess2)))
        except geocoders.google.GTooManyQueriesError as e:
            logger.error("%s %s" % (e.message, ", ".join(guess1)))


class System(ContactFieldsMixin):
    name = models.CharField(max_length=60)
    slug = models.SlugField(max_length=60, unique=True)

    def __unicode__(self):
        return self.name

    def get_absolute_url(self):
        return reverse('tx_highered:system_detail', kwargs={'slug': self.slug})


class WikipediaFields(models.Model):
    # wikipedia
    wikipedia_title = models.CharField(max_length=100, null=True, blank=True)
    wikipedia_abstract = models.TextField(null=True, blank=True)
    wikipedia_seal = models.ImageField(upload_to="%s/seals" % APP_LABEL,
        null=True, blank=True)
    wikipedia_logo = models.ImageField(upload_to="%s/logos" % APP_LABEL,
        null=True, blank=True)
    wikipedia_scraped = models.DateTimeField(null=True, blank=True)

    @property
    def wikipedia_url(self):
        if not self.wikipedia_title:
            return None
        return "http://en.wikipedia.org/w/index.php?title=%s" % (
            self.wikipedia_title.replace(" ", "_"))

    class Meta:
        abstract = True


class InstitutionManager(models.GeoManager):
    def published(self):
        """ only return institutions ready to be shown """
        qs = self.get_query_set()
        return qs.filter(Q(ipeds_id__isnull=False) | Q(fice_id__isnull=False)).\
            exclude(Q(institution_type='med') | Q(institution_type='pri_chi'))


class Institution(ContactFieldsMixin, WikipediaFields):
    name = models.CharField(max_length=60)
    slug = models.SlugField(max_length=60, unique=True)
    is_private = models.BooleanField(default=False)
    institution_type = models.CharField(max_length=30,
            choices=INSTITUTION_CHOICES, null=True, blank=True)
    # administrator officer
    system = models.ForeignKey(System, null=True, blank=True)
    description = models.TextField(null=True, blank=True)

    # WISHLIST change these institution identifiers to be charfields instead of
    # ints so we can do zero-padding because that's how they're reported,
    # except then it'll be harder to look up institutions because you'll have
    # to be careful about '0's.

    # Integrated Postsecondary Education Data System ID
    ipeds_id = models.IntegerField(null=True, blank=True)
    # Federal Interagency Committee on Education ID, used on the State Level
    fice_id = models.IntegerField(null=True, blank=True)
    # Office of Postsecondary Education ID
    # only Title IV schools have this. This is a 6 digit zero padded number with
    # a two digit suffix for each location/branch
    ope_id = models.IntegerField(null=True, blank=True)

    objects = InstitutionManager()

    def __unicode__(self):
        if self.system:
            return u"%s - %s" % (self.system, self.name)
        else:
            return self.name

    def get_absolute_url(self):
        return reverse(
            'tx_highered:institution_detail', kwargs={'slug': self.slug})

    #################### THECB / IPEDS ROUTERS #################
    def get_admissions(self):
        """
        Get admissions data. First, try IPEDS, then, try THECB.
        """
        # if not self.is_private and self.institution_type == 'uni':
        #     return self.publicadmissions.all()
        return self.admissions.all()

    def get_graduation_rates(self):
        """
        Public institutions use THECB graduation rates; others use IPEDS.
        """
        if not self.is_private:
            return self.publicgraduationrates.all()
        else:
            return self.graduationrates.all()

    @property
    def type(self):
        """ Get human readable version of `institution_type` """
        return dict(INSTITUTION_CHOICES).get(self.institution_type)

    @property
    def sentences(self):
        """ Get a generated sentence about the `Institution` """
        # TODO use cache backend (was caching in memory)
        return SummarySentences(self)

    # DELETEME replace with fuzzy matcher or can just delete
    @staticmethod
    def get_unique_name(name):
        ignored = ('st', 'saint', 'college', 'university', 'county', 'district', 'the', 'of', 'at')
        filtered_bits = [x for x in slugify(name).split('-') if x not in ignored]
        return ''.join(filtered_bits)

    # DELETEME replace with fuzzy matcher or can just delete
    @property
    def unique_name(self):
        return Institution.get_unique_name(self.name)

    @property
    def logo(self):
        """ return the `Image` that represents the logo or `None` """
        return self.wikipedia_logo

    @property
    def seal(self):
        """ return the `Image` that represents the seal or `None` """
        return self.wikipedia_seal

    @cached_property
    def enrollment_fte(self):
        """
        Quick stat to find out how many students are enrolled.

        Designed to answer the question, "so how many students are at ____?"
        Uses full time equivalent so community colleges with lots of part-time
        enrollment aren't inflated.
        """
        enrollment = self.enrollment.latest('year')
        return enrollment.fulltime_equivalent if enrollment else None

    @property
    def latest_tuition(self):
        return self.pricetrends.latest('year')

    @property
    def latest_enrollment(self):
        try:
            if self.publicenrollment.exists():
                return self.publicenrollment.latest('year')
            else:
                return self.enrollment.latest('year')
        except ObjectDoesNotExist:
            return None

    @property
    def sentence_name(self):
        if self.name.startswith('University'):
            return u'The %s' % self.name
        else:
            return self.name

    @property
    def sentence_institution_type(self):
        institution_type = self.get_institution_type_display().lower()
        if self.is_private:
            return u'private %s' % institution_type
        else:
            return u'public %s' % institution_type

    @property
    def geojson(self):
        from tx_highered.api import JSON
        if self.location:
            return JSON(self.location.geojson)
        else:
            return None

    @property
    def lege_districts(self):
        location = self.location
        if not location:
            return None

        return (District.objects.filter(geometry__contains=location.wkt)
                .exclude(type=SBOE))

    @property
    def districts_sentence(self):
        districts = self.lege_districts
        if districts:
            return DistrictsSentence(self, *districts)
        else:
            return None

    ############################## BUCKETS ##############################
    @cached_property
    def tuition_buckets(self):
        fields = ("in_state", "out_of_state", "books_and_supplies", "room_and_board_on_campus")
        return self.get_buckets('pricetrends', fields=fields)

    @cached_property
    def sat_score_buckets(self):
        b = {
            'years': [],
            'verbal_range': {},
            'math_range': {},
            'writing_range': {},
        }
        for a in self.testscores.all():
            b['years'].append(a.year)
            b['verbal_range'][a.year] = (a.sat_verbal_range
                    if a.sat_verbal_25th_percentile else 'N/A')
            b['math_range'][a.year] = (a.sat_math_range
                    if a.sat_math_25th_percentile else 'N/A')
            b['writing_range'][a.year] = (a.sat_writing_range
                    if a.sat_writing_25th_percentile else 'N/A')
        return b

    @cached_property
    def admission_buckets(self):
        b = {
            'years': [],
            'applicants': {},
            'admitted': {},
            'enrolled': {},
        }
        for a in self.get_admissions().all():
            if not a.number_admitted:
                continue
            b['years'].append(a.year)
            b['applicants'][a.year] = a.number_of_applicants
            b['admitted'][a.year] = (a.number_admitted,
                    a.percent_of_applicants_admitted)
            b['enrolled'][a.year] = (a.number_admitted_who_enrolled,
                    a.percent_of_admitted_who_enrolled)
        return b

    @cached_property
    def admission_top10_buckets(self):
        return self.get_buckets('admissions', fields=['percent_top10rule'],
                                filter_on_field='percent_top10rule')

    def get_buckets(self, relation_name, pivot_on_field="year",
                    filter_on_field=None, fields=None):
        """ pivot a related report model about the year field """
        b = defaultdict(dict)
        pivot_axis = []
        relation = getattr(self, relation_name)
        b['data_source'] = relation.model.data_source
        if fields is None:
            # XXX requires instacharts
            fields = [x[0] for x in relation.model.get_chart_series() if x[0] != pivot_on_field]
        for report_obj in relation.all():
            if filter_on_field is not None:
                # make sure we want this data
                filter_value = getattr(report_obj, filter_on_field)
                if filter_value is None or filter_value is '':
                    # null or blank
                    continue
            pivot = getattr(report_obj, pivot_on_field)
            pivot_axis.append(pivot)
            for field in fields:
                b[field][pivot] = getattr(report_obj, field)
        b[pivot_on_field + "s"] = pivot_axis  # poor man's `pluralize()`
        return b

    @cached_property
    def enrollment_buckets(self):
        """Get enrollment data from IPEDS."""
        b = self.get_buckets("enrollment",
            fields=("fulltime_equivalent", "fulltime", "parttime"))
        b['data_source'] = "IPEDS"
        return b

    @cached_property
    def demographics_buckets(self):
        """
        Get the most recent demographics data.

        Sometimes ipeds has better data, sometimes it's thecb. There isn't a
        magic way to figure out which is the latest without manually checking.

        XXX assumes school has white people.
        """
        try:
            latest_ipeds = self.enrollment.filter(
                total_percent_white__isnull=False).latest('year').year
        except ObjectDoesNotExist:
            latest_ipeds = 0
        try:
            latest_thecb = self.publicenrollment.filter(
                white_percent__isnull=False).latest('year').year
        except ObjectDoesNotExist:
            latest_thecb = 0
        if latest_ipeds > latest_thecb:
            return self.get_buckets("enrollment", filter_on_field="total_percent_white")
        return self.get_buckets("publicenrollment")

    @cached_property
    def graduationrates_buckets(self):
        if self.is_private:
            return self.get_buckets("graduationrates")
        else:
            return self.get_buckets("publicgraduationrates")



from django import template
from django.template.base import TemplateSyntaxError
from django.template.loader import render_to_string
from django.utils.safestring import mark_safe

from ..models import SimpleChart, ChartCell


DEFAULT_CHART_NAME = "simple_chart"

register = template.Library()


# top stuff is directly from armstrong layout_helpers with object renamed obj
@register.tag(name="instachart")
def do_render_qs(parser, token):
    tokens = token.split_contents()
    if len(tokens) is 2:
        tokens.append(u'"%s"' % DEFAULT_CHART_NAME)
    if len(tokens) is 3:
        _, obj, name = tokens
        return RenderQuerysetNode(obj, name)

    message = "Too %s parameters" % ("many" if len(tokens) > 3 else "few")
    raise TemplateSyntaxError(message)


class RenderQuerysetNode(template.Node):
    def __init__(self, qs, name):
        self.qs = template.Variable(qs)
        self.name = template.Variable(name)

    def render(self, context):
        qs = self.qs.resolve(context)
        name = self.name.resolve(context)
        return render_queryset(qs, name, dictionary={}, context_instance=context)


class ChartsRenderQuerysetBackend(object):
    def get_layout_template_name(self, obj, name):
        ret = []
        for a in obj.mro():
            if not hasattr(a, "_meta"):
                continue
            ret.append("instachart/%s/%s.html" % (a._meta.object_name.lower(), name))
        return ret

    def render(self, qs, name, dictionary=None, context_instance=None):
        dictionary = dictionary or {}
        dictionary["object_list"] = qs
        try:
            dictionary["chart_header"] = qs.model.get_chart_header()
            template_name = self.get_layout_template_name(qs.model, name)
        except AttributeError:
            fields = [x.name for x in qs.model._meta.fields]
            dictionary["chart_header"] = [ChartCell(qs.model, field) for field in fields]
            template_name = "instachart/simplechart/%s.html" % name
        return mark_safe(render_to_string(template_name, dictionary=dictionary,
            context_instance=context_instance))

    def __call__(self, *args, **kwargs):
        return self.render(*args, **kwargs)

render_queryset = ChartsRenderQuerysetBackend()


@register.filter(name="chart_set")
def chart_set(obj):
    return SimpleChart.chart_set(obj)



#! /usr/bin/env python
# -*- coding: UTF-8 -*-
"""
Automate migrating the data from launch state to what we need in 2014.

You only need to run this once, but it's safe to run repeatedly.
"""
from __future__ import unicode_literals

from tx_highered.models import Institution


def associate_districts():
    """
    Associate existing districts with ids so we can import to them.
    """
    institution = Institution.objects.get(slug='alamo-community-college-district')
    institution.fice_id = '003607'
    institution.ipeds_id = '222497'
    institution.save()

    institution = Institution.objects.get(slug='dallas-county-community-college-district')
    institution.fice_id = '009331'
    institution.ipeds_id = '224253'
    institution.save()

    institution = Institution.objects.get(slug='howard-county-junior-college-district')
    institution.fice_id = '103574'
    institution.save()

    institution = Institution.objects.get(slug='lone-star-college-system-district')
    institution.fice_id = '011145'
    # lone star already has an ipeds id
    institution.save()

    institution = Institution.objects.get(slug='san-jacinto-community-college')
    institution.fice_id = '029137'
    # San Jacinto College Central Campus has the ipeds id for the system
    institution.save()

    institution = Institution.objects.get(slug='tarrant-county-college-district')
    institution.fice_id = '003626'
    institution.ipeds_id = '228547'
    institution.save()


def add_new_institutions():
    Institution.objects.get_or_create(
        slug='lone-star-college-university-park',
        fice_id='000821',
        defaults=dict(
            name='Lone Star College - University Park',
            address='20515 SH 249',
            city='Houston',
            zip_code='77070-2607',
            phone='281-290-2600',
            url='http://www.lonestar.edu/universitypark.htm',
            location='POINT(-95.57932 29.99093)',
            institution_type='pub_cc',
            system_id=11,
        ),
    )


def fix_old_institutions():
    # This is a private law school, which is beyond the scope of this explorer
    institution = Institution.objects.get(slug='south-texas-college-of-law')
    institution.ipeds_id = None
    institution.save()


if __name__ == '__main__':
    add_new_institutions()
    associate_districts()
    fix_old_institutions()



from django.conf.urls import patterns, url
from django.views.generic import ListView

from .models import System
from .views import (HomeView,
                    InstitutionDetailView, InstitutionListView,
                    RenderModelDetailView,
                    TestScoresReport, FunnelReport, Top10RuleReport)
from . import api


urlpatterns = patterns('',
    url(r'^$', HomeView.as_view(), name="home"),
    url(r'^institutions/$', InstitutionListView.as_view(),
        name="institution_list"),
    url(r'^system/$', ListView.as_view(
        model=System), name="system_list"),
    url(r'^system/(?P<slug>[-\w]+)/$', RenderModelDetailView.as_view(
        model=System, layout="table"), name="system_detail"),
    # reports
    url(r'^testing/$', TestScoresReport.as_view(), name="sat_search"),
    url(r'^funnel/$', FunnelReport.as_view()),
    url(r'^top10/$', Top10RuleReport.as_view()),
    url(r'^(?P<slug>[-\w]+)/$', InstitutionDetailView.as_view(),
        name="institution_detail"),
) + patterns('tx_highered.api',
    url(r'^api/institution/(?P<pk>\d+)/enrollment/$', 'enrollment_api',
        name='enrollment_api'),
    url(r'^api/institution/(?P<pk>\d+)/(?P<metric>\w+)/$',
        api.ReportView.as_view(report_name='testscores'),
        name='institution_api'),
    url(r'^api/institution/$', 'institution_api', name='institution_api'),
)



# encoding: utf-8
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models

class Migration(SchemaMigration):

    def forwards(self, orm):
        
        # Adding model 'System'
        db.create_table('tx_highered_system', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('address', self.gf('django.db.models.fields.CharField')(max_length=200, null=True, blank=True)),
            ('city', self.gf('django.db.models.fields.CharField')(max_length=50, null=True, blank=True)),
            ('zip_code', self.gf('django.db.models.fields.CharField')(max_length=10, null=True, blank=True)),
            ('phone', self.gf('django.db.models.fields.CharField')(max_length=15, null=True, blank=True)),
            ('url', self.gf('django.db.models.fields.URLField')(max_length=200, null=True, blank=True)),
            ('name', self.gf('django.db.models.fields.CharField')(max_length=60)),
            ('slug', self.gf('django.db.models.fields.SlugField')(unique=True, max_length=60, db_index=True)),
        ))
        db.send_create_signal('tx_highered', ['System'])

        # Adding model 'Institution'
        db.create_table('tx_highered_institution', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('address', self.gf('django.db.models.fields.CharField')(max_length=200, null=True, blank=True)),
            ('city', self.gf('django.db.models.fields.CharField')(max_length=50, null=True, blank=True)),
            ('zip_code', self.gf('django.db.models.fields.CharField')(max_length=10, null=True, blank=True)),
            ('phone', self.gf('django.db.models.fields.CharField')(max_length=15, null=True, blank=True)),
            ('url', self.gf('django.db.models.fields.URLField')(max_length=200, null=True, blank=True)),
            ('wikipedia_title', self.gf('django.db.models.fields.CharField')(max_length=100, null=True, blank=True)),
            ('wikipedia_abstract', self.gf('django.db.models.fields.TextField')(null=True, blank=True)),
            ('wikipedia_seal', self.gf('django.db.models.fields.files.ImageField')(max_length=100, null=True, blank=True)),
            ('wikipedia_logo', self.gf('django.db.models.fields.files.ImageField')(max_length=100, null=True, blank=True)),
            ('wikipedia_scraped', self.gf('django.db.models.fields.DateTimeField')(null=True, blank=True)),
            ('name', self.gf('django.db.models.fields.CharField')(max_length=60)),
            ('slug', self.gf('django.db.models.fields.SlugField')(unique=True, max_length=60, db_index=True)),
            ('is_private', self.gf('django.db.models.fields.BooleanField')(default=False)),
            ('institution_type', self.gf('django.db.models.fields.CharField')(max_length=30, null=True, blank=True)),
            ('system', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['tx_highered.System'], null=True, blank=True)),
            ('description', self.gf('django.db.models.fields.TextField')(null=True, blank=True)),
            ('ipeds_id', self.gf('django.db.models.fields.IntegerField')(null=True, blank=True)),
            ('fice_id', self.gf('django.db.models.fields.IntegerField')(null=True, blank=True)),
            ('ope_id', self.gf('django.db.models.fields.IntegerField')(null=True, blank=True)),
        ))
        db.send_create_signal('tx_highered', ['Institution'])

        # Adding model 'PriceTrends'
        db.create_table('tx_highered_pricetrends', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('year', self.gf('django.db.models.fields.IntegerField')(default=1970)),
            ('year_type', self.gf('django.db.models.fields.CharField')(max_length=10, null=True)),
            ('institution', self.gf('django.db.models.fields.related.ForeignKey')(related_name='pricetrends', to=orm['tx_highered.Institution'])),
            ('tuition_fees_in_state', self.gf('django.db.models.fields.IntegerField')(null=True)),
            ('tuition_fees_outof_state', self.gf('django.db.models.fields.IntegerField')(null=True)),
            ('books_and_supplies', self.gf('django.db.models.fields.IntegerField')(null=True)),
            ('room_and_board_on_campus', self.gf('django.db.models.fields.IntegerField')(null=True)),
            ('room_and_board_off_campus', self.gf('django.db.models.fields.IntegerField')(null=True)),
            ('room_and_board_off_campus_w_family', self.gf('django.db.models.fields.IntegerField')(null=True)),
        ))
        db.send_create_signal('tx_highered', ['PriceTrends'])

        # Adding unique constraint on 'PriceTrends', fields ['year', 'institution']
        db.create_unique('tx_highered_pricetrends', ['year', 'institution_id'])

        # Adding model 'TestScores'
        db.create_table('tx_highered_testscores', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('year', self.gf('django.db.models.fields.IntegerField')(default=1970)),
            ('year_type', self.gf('django.db.models.fields.CharField')(max_length=10, null=True)),
            ('institution', self.gf('django.db.models.fields.related.ForeignKey')(related_name='testscores', to=orm['tx_highered.Institution'])),
            ('sat_verbal_25th_percentile', self.gf('django.db.models.fields.IntegerField')(null=True)),
            ('sat_verbal_75th_percentile', self.gf('django.db.models.fields.IntegerField')(null=True)),
            ('sat_math_25th_percentile', self.gf('django.db.models.fields.IntegerField')(null=True)),
            ('sat_math_75th_percentile', self.gf('django.db.models.fields.IntegerField')(null=True)),
            ('sat_writing_25th_percentile', self.gf('django.db.models.fields.IntegerField')(null=True)),
            ('sat_writing_75th_percentile', self.gf('django.db.models.fields.IntegerField')(null=True)),
            ('sat_submitted_number', self.gf('django.db.models.fields.IntegerField')(null=True)),
            ('sat_submitted_percent', self.gf('django.db.models.fields.IntegerField')(null=True)),
            ('act_composite_25th_percentile', self.gf('django.db.models.fields.IntegerField')(null=True)),
            ('act_composite_75th_percentile', self.gf('django.db.models.fields.IntegerField')(null=True)),
            ('act_english_25th_percentile', self.gf('django.db.models.fields.IntegerField')(null=True)),
            ('act_english_75th_percentile', self.gf('django.db.models.fields.IntegerField')(null=True)),
            ('act_math_25th_percentile', self.gf('django.db.models.fields.IntegerField')(null=True)),
            ('act_math_75th_percentile', self.gf('django.db.models.fields.IntegerField')(null=True)),
            ('act_writing_25th_percentile', self.gf('django.db.models.fields.IntegerField')(null=True)),
            ('act_writing_75th_percentile', self.gf('django.db.models.fields.IntegerField')(null=True)),
            ('act_submitted_number', self.gf('django.db.models.fields.IntegerField')(null=True)),
            ('act_submitted_percent', self.gf('django.db.models.fields.IntegerField')(null=True)),
        ))
        db.send_create_signal('tx_highered', ['TestScores'])

        # Adding unique constraint on 'TestScores', fields ['year', 'institution']
        db.create_unique('tx_highered_testscores', ['year', 'institution_id'])

        # Adding model 'Admissions'
        db.create_table('tx_highered_admissions', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('year', self.gf('django.db.models.fields.IntegerField')(default=1970)),
            ('year_type', self.gf('django.db.models.fields.CharField')(max_length=10, null=True)),
            ('institution', self.gf('django.db.models.fields.related.ForeignKey')(related_name='admissions', to=orm['tx_highered.Institution'])),
            ('number_of_applicants', self.gf('django.db.models.fields.IntegerField')(null=True, blank=True)),
            ('number_admitted', self.gf('django.db.models.fields.IntegerField')(null=True, blank=True)),
            ('number_admitted_who_enrolled', self.gf('django.db.models.fields.IntegerField')(null=True, blank=True)),
            ('percent_of_applicants_admitted', self.gf('django.db.models.fields.DecimalField')(null=True, max_digits=4, decimal_places=1, blank=True)),
            ('percent_of_admitted_who_enrolled', self.gf('django.db.models.fields.DecimalField')(null=True, max_digits=4, decimal_places=1, blank=True)),
            ('percent_top10rule', self.gf('django.db.models.fields.DecimalField')(null=True, max_digits=4, decimal_places=1)),
        ))
        db.send_create_signal('tx_highered', ['Admissions'])

        # Adding unique constraint on 'Admissions', fields ['year', 'institution']
        db.create_unique('tx_highered_admissions', ['year', 'institution_id'])

        # Adding model 'Enrollment'
        db.create_table('tx_highered_enrollment', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('year', self.gf('django.db.models.fields.IntegerField')(default=1970)),
            ('year_type', self.gf('django.db.models.fields.CharField')(max_length=10, null=True)),
            ('institution', self.gf('django.db.models.fields.related.ForeignKey')(related_name='enrollment', to=orm['tx_highered.Institution'])),
            ('total', self.gf('django.db.models.fields.IntegerField')(null=True)),
            ('fulltime_equivalent', self.gf('django.db.models.fields.IntegerField')(null=True)),
            ('fulltime', self.gf('django.db.models.fields.IntegerField')(null=True)),
            ('parttime', self.gf('django.db.models.fields.IntegerField')(null=True)),
            ('total_percent_white', self.gf('django.db.models.fields.IntegerField')(null=True)),
            ('total_percent_black', self.gf('django.db.models.fields.IntegerField')(null=True)),
            ('total_percent_hispanic', self.gf('django.db.models.fields.IntegerField')(null=True)),
            ('total_percent_native', self.gf('django.db.models.fields.IntegerField')(null=True)),
            ('total_percent_asian', self.gf('django.db.models.fields.IntegerField')(null=True)),
            ('total_percent_unknown', self.gf('django.db.models.fields.IntegerField')(null=True)),
        ))
        db.send_create_signal('tx_highered', ['Enrollment'])

        # Adding unique constraint on 'Enrollment', fields ['year', 'institution']
        db.create_unique('tx_highered_enrollment', ['year', 'institution_id'])

        # Adding model 'GraduationRates'
        db.create_table('tx_highered_graduationrates', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('year', self.gf('django.db.models.fields.IntegerField')(default=1970)),
            ('year_type', self.gf('django.db.models.fields.CharField')(max_length=10, null=True)),
            ('institution', self.gf('django.db.models.fields.related.ForeignKey')(related_name='graduationrates', to=orm['tx_highered.Institution'])),
            ('bachelor_4yr', self.gf('django.db.models.fields.IntegerField')(null=True)),
            ('bachelor_5yr', self.gf('django.db.models.fields.IntegerField')(null=True)),
            ('bachelor_6yr', self.gf('django.db.models.fields.IntegerField')(null=True)),
        ))
        db.send_create_signal('tx_highered', ['GraduationRates'])

        # Adding unique constraint on 'GraduationRates', fields ['year', 'institution']
        db.create_unique('tx_highered_graduationrates', ['year', 'institution_id'])


    def backwards(self, orm):
        
        # Removing unique constraint on 'GraduationRates', fields ['year', 'institution']
        db.delete_unique('tx_highered_graduationrates', ['year', 'institution_id'])

        # Removing unique constraint on 'Enrollment', fields ['year', 'institution']
        db.delete_unique('tx_highered_enrollment', ['year', 'institution_id'])

        # Removing unique constraint on 'Admissions', fields ['year', 'institution']
        db.delete_unique('tx_highered_admissions', ['year', 'institution_id'])

        # Removing unique constraint on 'TestScores', fields ['year', 'institution']
        db.delete_unique('tx_highered_testscores', ['year', 'institution_id'])

        # Removing unique constraint on 'PriceTrends', fields ['year', 'institution']
        db.delete_unique('tx_highered_pricetrends', ['year', 'institution_id'])

        # Deleting model 'System'
        db.delete_table('tx_highered_system')

        # Deleting model 'Institution'
        db.delete_table('tx_highered_institution')

        # Deleting model 'PriceTrends'
        db.delete_table('tx_highered_pricetrends')

        # Deleting model 'TestScores'
        db.delete_table('tx_highered_testscores')

        # Deleting model 'Admissions'
        db.delete_table('tx_highered_admissions')

        # Deleting model 'Enrollment'
        db.delete_table('tx_highered_enrollment')

        # Deleting model 'GraduationRates'
        db.delete_table('tx_highered_graduationrates')


    models = {
        'tx_highered.admissions': {
            'Meta': {'ordering': "['year']", 'unique_together': "(('year', 'institution'),)", 'object_name': 'Admissions'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'institution': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'admissions'", 'to': "orm['tx_highered.Institution']"}),
            'number_admitted': ('django.db.models.fields.IntegerField', [], {'null': 'True', 'blank': 'True'}),
            'number_admitted_who_enrolled': ('django.db.models.fields.IntegerField', [], {'null': 'True', 'blank': 'True'}),
            'number_of_applicants': ('django.db.models.fields.IntegerField', [], {'null': 'True', 'blank': 'True'}),
            'percent_of_admitted_who_enrolled': ('django.db.models.fields.DecimalField', [], {'null': 'True', 'max_digits': '4', 'decimal_places': '1', 'blank': 'True'}),
            'percent_of_applicants_admitted': ('django.db.models.fields.DecimalField', [], {'null': 'True', 'max_digits': '4', 'decimal_places': '1', 'blank': 'True'}),
            'percent_top10rule': ('django.db.models.fields.DecimalField', [], {'null': 'True', 'max_digits': '4', 'decimal_places': '1'}),
            'year': ('django.db.models.fields.IntegerField', [], {'default': '1970'}),
            'year_type': ('django.db.models.fields.CharField', [], {'max_length': '10', 'null': 'True'})
        },
        'tx_highered.enrollment': {
            'Meta': {'ordering': "['year']", 'unique_together': "(('year', 'institution'),)", 'object_name': 'Enrollment'},
            'fulltime': ('django.db.models.fields.IntegerField', [], {'null': 'True'}),
            'fulltime_equivalent': ('django.db.models.fields.IntegerField', [], {'null': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'institution': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'enrollment'", 'to': "orm['tx_highered.Institution']"}),
            'parttime': ('django.db.models.fields.IntegerField', [], {'null': 'True'}),
            'total': ('django.db.models.fields.IntegerField', [], {'null': 'True'}),
            'total_percent_asian': ('django.db.models.fields.IntegerField', [], {'null': 'True'}),
            'total_percent_black': ('django.db.models.fields.IntegerField', [], {'null': 'True'}),
            'total_percent_hispanic': ('django.db.models.fields.IntegerField', [], {'null': 'True'}),
            'total_percent_native': ('django.db.models.fields.IntegerField', [], {'null': 'True'}),
            'total_percent_unknown': ('django.db.models.fields.IntegerField', [], {'null': 'True'}),
            'total_percent_white': ('django.db.models.fields.IntegerField', [], {'null': 'True'}),
            'year': ('django.db.models.fields.IntegerField', [], {'default': '1970'}),
            'year_type': ('django.db.models.fields.CharField', [], {'max_length': '10', 'null': 'True'})
        },
        'tx_highered.graduationrates': {
            'Meta': {'ordering': "['year']", 'unique_together': "(('year', 'institution'),)", 'object_name': 'GraduationRates'},
            'bachelor_4yr': ('django.db.models.fields.IntegerField', [], {'null': 'True'}),
            'bachelor_5yr': ('django.db.models.fields.IntegerField', [], {'null': 'True'}),
            'bachelor_6yr': ('django.db.models.fields.IntegerField', [], {'null': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'institution': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'graduationrates'", 'to': "orm['tx_highered.Institution']"}),
            'year': ('django.db.models.fields.IntegerField', [], {'default': '1970'}),
            'year_type': ('django.db.models.fields.CharField', [], {'max_length': '10', 'null': 'True'})
        },
        'tx_highered.institution': {
            'Meta': {'object_name': 'Institution'},
            'address': ('django.db.models.fields.CharField', [], {'max_length': '200', 'null': 'True', 'blank': 'True'}),
            'city': ('django.db.models.fields.CharField', [], {'max_length': '50', 'null': 'True', 'blank': 'True'}),
            'description': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'fice_id': ('django.db.models.fields.IntegerField', [], {'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'institution_type': ('django.db.models.fields.CharField', [], {'max_length': '30', 'null': 'True', 'blank': 'True'}),
            'ipeds_id': ('django.db.models.fields.IntegerField', [], {'null': 'True', 'blank': 'True'}),
            'is_private': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '60'}),
            'ope_id': ('django.db.models.fields.IntegerField', [], {'null': 'True', 'blank': 'True'}),
            'phone': ('django.db.models.fields.CharField', [], {'max_length': '15', 'null': 'True', 'blank': 'True'}),
            'slug': ('django.db.models.fields.SlugField', [], {'unique': 'True', 'max_length': '60', 'db_index': 'True'}),
            'system': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['tx_highered.System']", 'null': 'True', 'blank': 'True'}),
            'url': ('django.db.models.fields.URLField', [], {'max_length': '200', 'null': 'True', 'blank': 'True'}),
            'wikipedia_abstract': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'wikipedia_logo': ('django.db.models.fields.files.ImageField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'wikipedia_scraped': ('django.db.models.fields.DateTimeField', [], {'null': 'True', 'blank': 'True'}),
            'wikipedia_seal': ('django.db.models.fields.files.ImageField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'wikipedia_title': ('django.db.models.fields.CharField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'zip_code': ('django.db.models.fields.CharField', [], {'max_length': '10', 'null': 'True', 'blank': 'True'})
        },
        'tx_highered.pricetrends': {
            'Meta': {'ordering': "['year']", 'unique_together': "(('year', 'institution'),)", 'object_name': 'PriceTrends'},
            'books_and_supplies': ('django.db.models.fields.IntegerField', [], {'null': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'institution': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'pricetrends'", 'to': "orm['tx_highered.Institution']"}),
            'room_and_board_off_campus': ('django.db.models.fields.IntegerField', [], {'null': 'True'}),
            'room_and_board_off_campus_w_family': ('django.db.models.fields.IntegerField', [], {'null': 'True'}),
            'room_and_board_on_campus': ('django.db.models.fields.IntegerField', [], {'null': 'True'}),
            'tuition_fees_in_state': ('django.db.models.fields.IntegerField', [], {'null': 'True'}),
            'tuition_fees_outof_state': ('django.db.models.fields.IntegerField', [], {'null': 'True'}),
            'year': ('django.db.models.fields.IntegerField', [], {'default': '1970'}),
            'year_type': ('django.db.models.fields.CharField', [], {'max_length': '10', 'null': 'True'})
        },
        'tx_highered.system': {
            'Meta': {'object_name': 'System'},
            'address': ('django.db.models.fields.CharField', [], {'max_length': '200', 'null': 'True', 'blank': 'True'}),
            'city': ('django.db.models.fields.CharField', [], {'max_length': '50', 'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '60'}),
            'phone': ('django.db.models.fields.CharField', [], {'max_length': '15', 'null': 'True', 'blank': 'True'}),
            'slug': ('django.db.models.fields.SlugField', [], {'unique': 'True', 'max_length': '60', 'db_index': 'True'}),
            'url': ('django.db.models.fields.URLField', [], {'max_length': '200', 'null': 'True', 'blank': 'True'}),
            'zip_code': ('django.db.models.fields.CharField', [], {'max_length': '10', 'null': 'True', 'blank': 'True'})
        },
        'tx_highered.testscores': {
            'Meta': {'ordering': "['year']", 'unique_together': "(('year', 'institution'),)", 'object_name': 'TestScores'},
            'act_composite_25th_percentile': ('django.db.models.fields.IntegerField', [], {'null': 'True'}),
            'act_composite_75th_percentile': ('django.db.models.fields.IntegerField', [], {'null': 'True'}),
            'act_english_25th_percentile': ('django.db.models.fields.IntegerField', [], {'null': 'True'}),
            'act_english_75th_percentile': ('django.db.models.fields.IntegerField', [], {'null': 'True'}),
            'act_math_25th_percentile': ('django.db.models.fields.IntegerField', [], {'null': 'True'}),
            'act_math_75th_percentile': ('django.db.models.fields.IntegerField', [], {'null': 'True'}),
            'act_submitted_number': ('django.db.models.fields.IntegerField', [], {'null': 'True'}),
            'act_submitted_percent': ('django.db.models.fields.IntegerField', [], {'null': 'True'}),
            'act_writing_25th_percentile': ('django.db.models.fields.IntegerField', [], {'null': 'True'}),
            'act_writing_75th_percentile': ('django.db.models.fields.IntegerField', [], {'null': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'institution': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'testscores'", 'to': "orm['tx_highered.Institution']"}),
            'sat_math_25th_percentile': ('django.db.models.fields.IntegerField', [], {'null': 'True'}),
            'sat_math_75th_percentile': ('django.db.models.fields.IntegerField', [], {'null': 'True'}),
            'sat_submitted_number': ('django.db.models.fields.IntegerField', [], {'null': 'True'}),
            'sat_submitted_percent': ('django.db.models.fields.IntegerField', [], {'null': 'True'}),
            'sat_verbal_25th_percentile': ('django.db.models.fields.IntegerField', [], {'null': 'True'}),
            'sat_verbal_75th_percentile': ('django.db.models.fields.IntegerField', [], {'null': 'True'}),
            'sat_writing_25th_percentile': ('django.db.models.fields.IntegerField', [], {'null': 'True'}),
            'sat_writing_75th_percentile': ('django.db.models.fields.IntegerField', [], {'null': 'True'}),
            'year': ('django.db.models.fields.IntegerField', [], {'default': '1970'}),
            'year_type': ('django.db.models.fields.CharField', [], {'max_length': '10', 'null': 'True'})
        }
    }

    complete_apps = ['tx_highered']



from django.contrib import admin

from django_object_actions import DjangoObjectActions

from .models import Institution, System


class InstitutionAdmin(DjangoObjectActions, admin.ModelAdmin):
    list_display = ('name', 'ipeds_id', 'fice_id', 'ope_id')
    list_filter = ('institution_type', 'is_private')
    # list_editable = ('ipeds_id', 'fice_id', 'ope_id')
    ordering = ('name', )

    def geocode(self, request, obj):
        old_location = obj.location
        obj.guess_location()
        if old_location != obj.location:
            obj.save()
            self.message_user(request, "Location Updated To: %s" % obj.location)
        else:
            self.message_user(request, "Location Unchanged")
    geocode.short_description = u"Remember to save before using"

    objectactions = ['geocode']

admin.site.register(Institution, InstitutionAdmin)
admin.site.register(System, admin.ModelAdmin)



from django.db import models


NULL_DISPLAY = "&ndash;"


class ChartCell(object):
    head_attrs = {}
    body_attrs = {}
    label = ""
    format = "%s"
    text = ""
    raw_text = ""

    def __init__(self, cls, fieldname, format=None):
        self.label = fieldname
        try:
            self.field = cls._meta.get_field(fieldname)
            self.text = self.field.verbose_name
        except models.FieldDoesNotExist:
            try:
                self.text = getattr(cls, fieldname).verbose_name
            except AttributeError:
                self.text = fieldname
        self.raw_text = self.text
        if format is not None:
            self.format = format
        if hasattr(cls, 'chart_head_attrs'):
            self.head_attrs = dict(cls.chart_head_attrs)
        if hasattr(cls, 'chart_body_attrs'):
            self.body_attrs = dict(cls.chart_body_attrs)

    def apply_format(self, template):
        try:
            return template % self.raw_text
        except TypeError:
            return template

    def build_attrs(self, attrs, label):
        if label not in attrs:
            return ""
        attr = attrs[label]
        if isinstance(attr, basestring):
            return self.apply_format(attr)
        return u" ".join(map(self.apply_format, attr))

    def as_th(self):
        # TODO get mark_safe to work
        # from django.utils.safestring import mark_safe
        if self.head_attrs and self.label in self.head_attrs:
            return u"<th %s>%s</th>" % (self.build_attrs(self.head_attrs, self.label), self.text)
        return u"<th>%s</th>" % self.text

    def as_td(self):
        if self.body_attrs and self.label in self.body_attrs:
            return u"<td %s>%s</td>" % (self.build_attrs(self.body_attrs, self.label), self.text)
        return u"<td>%s</td>" % self.text

    def as_text(self):
        return self.text

    def __repr__(self):
        return self.as_th()


class ChartBodyCell(ChartCell):
    def __init__(self, obj, fieldname, format=None):
        super(ChartBodyCell, self).__init__(obj, fieldname, format)
        self.raw_text = getattr(obj, fieldname)
        if hasattr(self.raw_text, '__call__'):
            self.raw_text = self.raw_text()
        if self.raw_text is None:
            self.text = NULL_DISPLAY
        else:
            self.text = self.format % self.raw_text

    def as_td_data(self):
        return u"<td data-value=\"%s\">%s</td>" % (self.raw_text, self.text)


class SimpleChart(models.Model):
    """ Model mixin that enables quick dumps via a template tag """
    chart_series = []

    chart_excluded_fields = ('id',)

    class Meta:
        abstract = True

    @classmethod
    def get_chart_field_names(cls):
        return [x.name for x in cls._meta.fields if x.name not in cls.chart_excluded_fields]

    @classmethod
    def get_chart_series(cls):
        if cls.chart_series:
            # normalize, wrap in tuple if a series was defined simple
            return [(x,) if isinstance(x, basestring) else x for x in cls.chart_series]
        return [(x, "%s") for x in cls.get_chart_field_names()]

    @classmethod
    def get_chart_header(cls):
        return [ChartCell(cls, *x) for x in cls.get_chart_series()]

    @staticmethod
    def chart_set(obj):
        # TODO pep-0378, needs python 2.7
        try:
            cells = [ChartBodyCell(obj, *x) for x in obj.get_chart_series()]
        except AttributeError:
            fields = [x.name for x in obj._meta.fields]
            cells = [ChartBodyCell(obj, field) for field in fields]
        return cells



"""
Usage:

    python load_enrollment.py

Loads enrollment data from THECB's PREP online site:
http://reports.thecb.state.tx.us/approot/dwprodrpt/enrmenu.htm

NOTE: the total enrollment number is not a meaningful figure except for
boasting purposes. The full time equivalent figure is a better measure of
student population, which comes from the Accountability system, not PREP.
"""
from collections import defaultdict
import logging

from pyquery import PyQuery as pq
import requests

from tx_highered.models import Institution, PublicEnrollment
from tx_highered.thecb_importer.utils import create_or_update


REPORT_URL = "http://reports.thecb.state.tx.us/ibi_apps/WFServlet?"

CURRENT_YEAR = 2014  # TODO pull the current year directly from the data

CATEGORY_DATA = {
    'public': {
        'instcateg': 'Public University',
    },
    'public 2-year': {
        'instcateg': 'Public 2-Year College',
    },
}

NAME_CLASSES = ['x10_0', 'x10_1']
FICE_CLASSES = ['x11_0', 'x11_1']
ETHNICITY_CLASSES = ['x12_0', 'x12_1']
DATA_CLASSES = ['x13_0', 'x13_1', 'x14_0', 'x14_1']
KNOWN_CLASSES = NAME_CLASSES + FICE_CLASSES + ETHNICITY_CLASSES + DATA_CLASSES

TOTAL_CLASS = 'x15'
KNOWN_CLASSES.append(TOTAL_CLASS)


logger = logging.getLogger(__name__)


def generate_payload(extra_data):
    return dict({
        'IBIC_server': 'EDASERVE',
        'IBIF_ex': 'enrethnicinst',
        'HIGHER_ED_RPT_SEM_NAME': 'Fall',
        'HIGHER_ED_RPT_YEAR': 'FOC_NONE',
        'instcode': 'FOC_NONE',
        'WFFMT': 'HTML',
        'IBIMR_Random': '0.9490816127508879',
        'instname': 'FOC_NONE',
    }, **extra_data)


def get_institutions(category):
    logger.debug('get_institutions - {}'.format(category))
    institutions = []

    category_data = CATEGORY_DATA[category]
    payload = generate_payload(category_data)
    response = requests.get(REPORT_URL, params=payload)
    doc = pq(response.content)

    current_data = {}
    fice = None
    current_year = None
    current_name = None
    for tr in doc.find('tr'):
        name = None
        current_year = CURRENT_YEAR
        for td in tr.getchildren():
            td_class = td.attrib.get('class')
            if td_class not in KNOWN_CLASSES or 'colspan' in td.attrib:
                continue

            if td_class in NAME_CLASSES:
                name = td.text.strip()
                if name and name != current_name:
                    if current_name:
                        institutions.append({
                            'name': current_name,
                            'fice': fice or None,
                            'data': current_data
                        })
                    current_data = {}
                    current_name = name
            elif td_class in FICE_CLASSES:
                if td.text.strip():
                    fice = td.text.strip()
            elif td_class in ETHNICITY_CLASSES:
                if td.text.strip():
                    ethnicity = td.text.strip()
            elif td_class == TOTAL_CLASS:
                current_data.setdefault('total', {})
                current_data['total'][current_year] = td.text.strip()
                current_year = current_year - 1
            else:
                current_data.setdefault(ethnicity, {})
                current_data[ethnicity][current_year] = td.text.strip()
                current_year = current_year - 1

    return institutions


def clean_field_name(field):
    return {
        'African American': 'african_american_count',
        'American Indian/Alaskan Native': 'native_american_count',
        'Asian': 'asian_count',
        'Hispanic': 'hispanic_count',
        'International': 'international_count',
        'Multiracial': 'multiracial_count',
        'Native Hawaiian/Pacific Island': 'pacific_islander_count',
        'Unknown or Not Reported': 'unknown_count',
        'White': 'white_count',
        'total': 'total'
    }[field]


def clean_institution_data(data):
    # Look up related institution from the FICE ID
    try:
        institution = Institution.objects.get(fice_id=data['fice'])
    except Institution.DoesNotExist:
        logger.warn('missing FICE ID %(fice)s (%(name)s)\n' % data)
        return

    # The data item looks like this (multiple ethnicities with a total):
    # { race: [{year: enrollment}, ...], ..., total: [...] }
    data_by_year = defaultdict(dict)
    for field, value in data['data'].items():
        field_name = clean_field_name(field)
        for year, enrollment in value.items():
            data_by_year[year][field_name] = int(enrollment.replace(',', ''))

    # Calculate percentages
    for year, year_data in data_by_year.items():
        total = year_data['total']
        if total != 0:
            for field, count in year_data.items():
                if field == 'total':
                    continue
                percent_field = field.replace('_count', '_percent')
                year_data[percent_field] = round(100.0 * count / total, 1)
        yield dict(year_data, institution=institution, year=year)


def main():
    for category in ('public', 'public 2-year'):
        for institution in get_institutions(category):
            for cleaned_data in clean_institution_data(institution):
                inst = cleaned_data.pop('institution')
                year = cleaned_data.pop('year')
                logger.info('Instutition: {}, Year: {}'.format(inst, year))
                logger.debug('data: {}'.format(cleaned_data))
                create_or_update(PublicEnrollment.objects, institution=inst,
                    year=year, defaults=cleaned_data)


if __name__ == '__main__':
    main()



"""
Import the list of current public institutions
source: http://www.txhighereddata.org/Interactive/Institutions.cfm

"""
import urllib2

from django.template.defaultfilters import slugify

from tx_highered.models import Institution, System
from . import utils

# CONFIGURATION
SOURCE = "http://www.txhighereddata.org/Interactive/Institutionsshow_Excel.cfm?All=1"
INSTITUTION_TYPES = (
    ("pub_u", "http://www.txhighereddata.org/Interactive/InstitutionsShow_Excel.cfm?Level=1&Type=1"),
    ("pub_cc", "http://www.txhighereddata.org/Interactive/InstitutionsShow_Excel.cfm?Level=2&Type=1"),
    ("pub_med", "http://www.txhighereddata.org/Interactive/InstitutionsShow_Excel.cfm?Level=3&Type=1"),
    ("pub_tech", "http://www.txhighereddata.org/Interactive/InstitutionsShow_Excel.cfm?Level=7&Type=1"),
    ("pub_state", "http://www.txhighereddata.org/Interactive/InstitutionsShow_Excel.cfm?Level=11&Type=1"),
    ("pri_u", "http://www.txhighereddata.org/Interactive/InstitutionsShow_Excel.cfm?Level=1&Type=2"),
    ("pri_jr", "http://www.txhighereddata.org/Interactive/InstitutionsShow_Excel.cfm?Level=9&Type=2"),
    ("pri_med", "http://www.txhighereddata.org/Interactive/InstitutionsShow_Excel.cfm?Level=3&Type=2"),
    ("pri_chi", "http://www.txhighereddata.org/Interactive/InstitutionsShow_Excel.cfm?Level=10&Type=2"),
)


# institution header to fieldname map
I_MAP = {'Institution Name': 'name',
         'System Name': 'system__name',
         'Address': 'address',
         'City': 'city',
         'Zip Code': 'zip_code',
         'Administrative Officer': 'TODO',
         'Administrative Officer Title': 'TODO',
         'MainTelephone': 'phone',
         'Website Address': 'url'}


def pull(institution_type, source):
    r = urllib2.urlopen(source)
    reader = utils.DictReader(r, mapping=I_MAP)

    for data in reader:
        if not data['name']:
            continue
        data.pop('TODO')
        slug = slugify(data['name'])
        data['institution_type'] = institution_type
        system_name = data.pop('system__name')
        if system_name:
            system, _ = System.objects.get_or_create(slug=slugify(system_name), name=system_name)
        else:
            system = None
        data['system'] = system
        i, c = Institution.objects.get_or_create(slug=slug, defaults=data)
        print i, c


def main():
    for t, url in INSTITUTION_TYPES:
        pull(t, url)


if __name__ == "__main__":
    main()



#! /usr/bin/env python
# -*- coding: UTF-8 -*-
"""
Import from the Accountability report system.

Usage:
    ./import_thecb_report.py CSV1 CSV2 ...

Modeled after import_customreport.py, but written lazier.
"""
from collections import namedtuple
from csv import reader
import logging
import os
import re
import sys

from tx_highered.models import Institution, Admissions, PublicGraduationRates


logger = logging.getLogger(__name__)

# Similar to IPEDSCell
THECBCell = namedtuple('THECBCell', 'long_name, year, year_type')

ReportDatum = namedtuple('ReportDatum', ['model', 'field'])
# Field mappings, formatting is funny but PEP8. Not how I would have liked to
# have done it but oh well.
FIELD_MAPPINGS = {
    # Not used
    # u'First-Time Students in Top 10% (Percent)':
    # ReportDatum(Admissions, 'percent_top10rule'),

    # Graduation Rates - Universities
    u'Four-Year Graduation Rate - Percent Total (Rate)': ReportDatum(
        PublicGraduationRates, 'bachelor_4yr'),
    u'Five-Year Graduation Rate - Percent Total (Rate)': ReportDatum(
        PublicGraduationRates, 'bachelor_5yr'),
    u'Six-Year Graduation Rate - Percent Total (Rate)': ReportDatum(
        PublicGraduationRates, 'bachelor_6yr'),
    # Graduation Rates - Community College
    u'Three-Year Graduation Rate - Associates': ReportDatum(
        PublicGraduationRates, 'associate_3yr'),
    u'Three-Year Graduation Rate - Bachelors': ReportDatum(
        PublicGraduationRates, 'bachelor_3yr'),
    u'Four-Year Graduation Rate - Associates': ReportDatum(
        PublicGraduationRates, 'associate_4yr'),
    u'Four-Year Graduation Rate - Bachelors': ReportDatum(
        PublicGraduationRates, 'bachelor_4yr'),
    u'Six-Year Graduation Rate - Bachelors': ReportDatum(
        PublicGraduationRates, 'bachelor_6yr'),
    u'Six-Year Graduation Rate - Associates': ReportDatum(
        PublicGraduationRates, 'associate_6yr'),

}
# based on reports.YearBasedInstitutionStatsModel.YEAR_TYPE_CHOICES
YEAR_TYPES = {
    'Fall': 'fall',
    'FY': 'fiscal',
    # academic
    # calendar
    # aug
}


def parse_header_cell(text):
    """Get details needed for THECBCell out of the text."""
    search = re.match(r'(.+)\((\w+)\s(\d{4})\)', text)
    if search:
        name_raw, year_type_raw, year_raw = search.groups()
        year_type = YEAR_TYPES[year_type_raw]
        return THECBCell(name_raw.strip(), int(year_raw), year_type)
    return text


def generic(path):
    report = reader(open(path))
    original_header = report.next()
    header = map(parse_header_cell, original_header)

    for row in report:
        data = zip(header, row)
        fice_id = row[1]
        try:
            institution = Institution.objects.get(fice_id=fice_id)
        except Institution.DoesNotExist:
            logger.error('MISSING Institution: {} fice: {}'.format(
                row[0], fice_id,
            ))
            continue

        for key, value in data[3:]:
            if value.upper() in ('', 'NA'):
                # HACK so bad values get stored as NULL. NULl means we tried to
                # get data but got nothing or bad data.
                value = None
            logger.debug(u'{} {}'.format(key, value))
            try:
                finder = FIELD_MAPPINGS[key.long_name]
            except KeyError:
                logger.error('MISSING: cannot interpret {}'
                    .format(key.long_name))
                continue
            defaults = {
                finder.field: value,
                'year_type': key.year_type,
            }
            logging_state = 'CREATED'
            instance, created = finder.model.objects.get_or_create(
                institution=institution, year=key.year,
                defaults=defaults)
            if not created:
                if unicode(getattr(instance, finder.field)) != value:
                    logging_state = 'UPDATED'
                    instance.__dict__.update(defaults)
                    instance.save()
                else:
                    logging_state = 'SKIP'
            logger.info(u'{} {} {}'
                .format(instance, key.long_name, logging_state))


if __name__ == '__main__':
    for path in sys.argv[1:]:
        if os.path.isfile(path):
            generic(path)



import json

from django.http import HttpResponse
from django.views.generic import View
from django.views.generic.detail import SingleObjectMixin

from tx_highered.models import Institution


class JSON(int):
    """
    Wraps encoded JSON and ensures it passes through the default encoder
    without being encoded again.

    Relies on an implementation detail of the built-in json module. int
    values are encoded by calling `__str__` on them, so if that changes,
    this code will break.
    """
    def __new__(cls, content):
        o = super(JSON, cls).__new__(cls)
        o.content = str(content)
        return o

    def __str__(self):
        return self.content


class ApiView(View):
    def get(self, request, *args, **kwargs):
        data = self.get_content_data()
        content = json.dumps(data)
        return HttpResponse(content, content_type='application/json')


class EnrollmentApiView(SingleObjectMixin, ApiView):
    model = Institution

    def get_content_data(self):
        self.object = self.get_object()

        # Use IPEDS enrollment data if this is a private university;
        # otherwise use THECB data.
        if self.object.is_private:
            qs = self.object.enrollment.all()
        else:
            qs = self.object.publicenrollment.all()

        race_data = []
        for enrollment in qs:
            race_data.extend(enrollment.race_data())

        return race_data


class ReportView(SingleObjectMixin, ApiView):
    model = Institution
    report_name = None

    def get_content_data(self):
        self.object = self.get_object()
        return_data = []
        for obj in getattr(self.object, self.report_name).all():
            return_data.append(obj.__json__())
        return return_data


class InstitutionApiView(ApiView):
    available_fields = ['is_private', 'enrollment_fte', 'city', 'geojson']
    queryset = Institution.objects.published()

    def get_content_data(self):
        data = []
        for i in self.queryset:
            object_data = {'uri': i.get_absolute_url(), 'name': i.name}
            for field in self.request.GET.get('fields', '').split(','):
                if field in self.available_fields:
                    object_data[field] = getattr(i, field)
            data.append(object_data)

        return data


enrollment_api = EnrollmentApiView.as_view()
institution_api = InstitutionApiView.as_view()



"""
Import IPEDS Custom Reports.

Usage:
  ./import_customreport <csv1> <csv2> ...
"""
from collections import namedtuple
import logging
import os
import sys

from tx_highered.ipeds_csv_reader import IpedsCSVReader
from tx_highered.models import (
    Institution, Admissions, PriceTrends, TestScores, Enrollment,
    GraduationRates,
)


# This tells the importer how to interpret IPEDS short variables for import
# into a Django model and field
ReportDatum = namedtuple('ReportDatum', ['model', 'field', 'year_type'])
FIELD_MAPPINGS = {
    'ADMSSN': ReportDatum(Admissions, 'number_admitted', 'fall'),
    'APPLCN': ReportDatum(Admissions, 'number_of_applicants', 'fall'),
    'ENRLT': ReportDatum(Admissions, 'number_admitted_who_enrolled', 'fall'),
    # Price Trends
    'chg2ay3': ReportDatum(PriceTrends, 'tuition_fees_in_state', 'fall'),
    'chg3ay3': ReportDatum(PriceTrends, 'tuition_fees_outof_state', 'fall'),
    'chg4ay3': ReportDatum(PriceTrends, 'books_and_supplies', 'fall'),
    'chg5ay3': ReportDatum(PriceTrends, 'room_and_board_on_campus', 'fall'),

    # Test Scores
    'SATNUM': ReportDatum(TestScores, 'sat_submitted_number', 'fall'),
    'SATPCT': ReportDatum(TestScores, 'sat_submitted_percent', 'fall'),
    'ACTNUM': ReportDatum(TestScores, 'act_submitted_number', 'fall'),
    'ACTPCT': ReportDatum(TestScores, 'act_submitted_percent', 'fall'),
    'SATVR25': ReportDatum(TestScores, 'sat_verbal_25th_percentile', 'fall'),
    'SATVR75': ReportDatum(TestScores, 'sat_verbal_75th_percentile', 'fall'),
    'SATMT25': ReportDatum(TestScores, 'sat_math_25th_percentile', 'fall'),
    'SATMT75': ReportDatum(TestScores, 'sat_math_75th_percentile', 'fall'),
    'SATWR25': ReportDatum(TestScores, 'sat_writing_25th_percentile', 'fall'),
    'SATWR75': ReportDatum(TestScores, 'sat_writing_75th_percentile', 'fall'),
    'ACTCM25': ReportDatum(TestScores, 'act_composite_25th_percentile', 'fall'),
    'ACTCM75': ReportDatum(TestScores, 'act_composite_75th_percentile', 'fall'),
    'ACTEN25': ReportDatum(TestScores, 'act_english_25th_percentile', 'fall'),
    'ACTEN75': ReportDatum(TestScores, 'act_english_75th_percentile', 'fall'),
    'ACTMT25': ReportDatum(TestScores, 'act_math_25th_percentile', 'fall'),
    'ACTMT75': ReportDatum(TestScores, 'act_math_75th_percentile', 'fall'),
    'ACTWR25': ReportDatum(TestScores, 'act_writing_25th_percentile', 'fall'),
    'ACTWR75': ReportDatum(TestScores, 'act_writing_75th_percentile', 'fall'),

    # Enrollment
    'PctEnrWh': ReportDatum(Enrollment, 'total_percent_white', 'fall'),
    'PctEnrBK': ReportDatum(Enrollment, 'total_percent_black', 'fall'),
    'PctEnrHS': ReportDatum(Enrollment, 'total_percent_hispanic', 'fall'),
    'PctEnrAP': ReportDatum(Enrollment, 'total_percent_asian', 'fall'),
    'PctEnrAN': ReportDatum(Enrollment, 'total_percent_native', 'fall'),
    'PctEnrUn': ReportDatum(Enrollment, 'total_percent_unknown', 'fall'),
    'ENRTOT': ReportDatum(Enrollment, 'total', 'fall'),
    'FTE': ReportDatum(Enrollment, 'fulltime_equivalent', 'fall'),
    'EnrFt': ReportDatum(Enrollment, 'fulltime', 'fall'),
    'EnrPt': ReportDatum(Enrollment, 'parttime', 'fall'),

    # GraduationRates
    'GBA4RTT': ReportDatum(GraduationRates, 'bachelor_4yr', 'aug'),
    'GBA5RTT': ReportDatum(GraduationRates, 'bachelor_5yr', 'aug'),
    'GBA6RTT': ReportDatum(GraduationRates, 'bachelor_6yr', 'aug'),
}


def generic(path):
    """Read the CSV and import into the appropriate model."""
    logger = logging.getLogger(__name__)
    reader = IpedsCSVReader(open(path, 'rb'))
    for row in reader:
        unit_id, ipeds_id = row[0]
        assert unit_id == 'UnitID'
        try:
            institution = Institution.objects.get(ipeds_id=ipeds_id)
        except Institution.MultipleObjectsReturned:
            logger.critical('DUPLICATE IPEDS: {}'.format(ipeds_id))
            sys.exit()
        except Institution.DoesNotExist:
            # TODO echo the name too
            logger.error('MISSING Institution: {}'.format(ipeds_id))
            continue
        for key, value in row[2:]:
            logger.debug(u'{} {}'.format(key, value))
            if key is None or value is '':
                # skip cells with no data, CSVs will give empty strings for
                # missing values
                continue
            try:
                finder = FIELD_MAPPINGS[key.short_name]
            except KeyError:
                logger.error('MISSING: cannot interpret {}'
                    .format(key.short_name))
                continue
            defaults = {
                finder.field: value,
                'year_type': finder.year_type,
            }
            logging_state = 'CREATED'
            instance, created = finder.model.objects.get_or_create(
                institution=institution, year=key.year,
                defaults=defaults)
            if not created:
                if unicode(getattr(instance, finder.field)) != value:
                    logging_state = 'UPDATED'
                    instance.__dict__.update(defaults)
                    instance.save()
                else:
                    logging_state = 'SKIP'
            logger.info(u'{} {} {}'
                .format(instance, key.short_name, logging_state))


if __name__ == '__main__':
    for path in sys.argv[1:]:
        if os.path.isfile(path):
            generic(path)



from django.views.generic import DetailView, ListView, TemplateView

from armstrong.core.arm_layout.utils import get_layout_template_name

from .models import Institution, Enrollment


# Enrollment data trickles in at different times, so pick the latest year where
# most institutions have data available.
LATEST_ENROLLMENT_YEAR = 2012


class RenderModelDetailView(DetailView):
    """ shortcut to rendering an object using render_model """
    layout = None

    def get_template_names(self):
        return get_layout_template_name(self.object, self.layout)


class FunnelMixin(object):
    """ Adds `funnels` property to an `Institution` for the Funnel viz """
    def annotate_funnels(self, inst):
        enterdata = inst.admissions.all()

        # please excuse the horribleness of this
        funnels = []
        for year in enterdata:
            try:
                funnel = {'year': year.year}
                funnel[0] = 100
                funnel[1] = float(year.percent_of_applicants_admitted)
                funnel[2] = funnel[1] * float(year.percent_of_admitted_who_enrolled) / 100
                funnels.append(funnel)
            except TypeError:
                continue
        inst.funnels = funnels


class HomeView(TemplateView):
    template_name = "tx_highered/home.html"

    def get_short_list(self):
        """
        List of highest enrollment schools to make getting to UT one click.

        Used full time equivalent, which means we can't use THECB data since
        getting it from them is difficult.

        To see which institutions are missing data, and to see what the latest
        safe year to use, run this command:

            for x in Institution.objects.published():
                try:
                    e = x.enrollment.latest('year')
                    print e.year, x, e.fulltime_equivalent
                except Enrollment.DoesNotExist:
                    print '!!!!', x, 'MISSING'
        """
        queryset = (Enrollment.objects.filter(year=LATEST_ENROLLMENT_YEAR)
            .select_related('institution')
            .order_by('-fulltime_equivalent')[:15])
        return queryset


class InstitutionListView(ListView):
    queryset = Institution.objects.filter(ipeds_id__isnull=False).\
        order_by('name')


class InstitutionDetailView(DetailView):
    model = Institution


######################### REPORTS ##########################
class TestScoresReport(InstitutionListView):
    template_name = "tx_highered/reports/testscores.html"

    def get_queryset(self):
        qs = super(TestScoresReport, self).get_queryset()
        qs = qs.exclude(testscores__isnull=True).order_by('name')
        for x in qs:
            x.scores = x.testscores.latest('year')
        return qs


class FunnelReport(InstitutionListView, FunnelMixin):
    template_name = "tx_highered/reports/funnel.html"

    def get_context_data(self, *args, **kwargs):
        # TODO this shouldn't be get_context_data
        context = super(FunnelReport, self).get_context_data(*args, **kwargs)
        for obj in self.object_list:
            self.annotate_funnels(obj)
        return context


class Top10RuleReport(InstitutionListView, FunnelMixin):
    template_name = "tx_highered/reports/top10.html"
    year_range = range(2000, 2012)

    def build_table(self, obj):
        raw_data = obj.admissions.filter(percent_top10rule__isnull=False).\
            values('year', 'percent_top10rule')
        data = dict([(x['year'], x['percent_top10rule']) for x in raw_data])
        return data

    def get_queryset(self):
        qs = super(Top10RuleReport, self).get_queryset()
        qs = qs.filter(institution_type='uni', is_private=False)
        # TODO prefetch admissions
        for x in qs:
            x.data = self.build_table(x)
        return qs

    def get_context_data(self, **kwargs):
        context = super(Top10RuleReport, self).get_context_data(**kwargs)
        context['year_range'] = self.year_range
        return context



#! /usr/bin/env python
#
# DEPRECATED in favor of just using the IPEDS data to avoid having to deal with
# scraping PDFs. We can't automate this, and it's costing us too much time to
# wrangle with this data source.
#
# The admissions data is loaded from PDF reports on the THECB website.
#
# Use pdftohtml to preprocess:
#
# find . -name "*.pdf" -exec sh -c 'pdftohtml -i -noframes -stdout "$1" > "$1.html"' -- {} \;
#
import glob
import HTMLParser
import os
import re
import sys
from collections import defaultdict
from decimal import Decimal
from pprint import pprint

from tx_highered.models import PublicAdmissions
from tx_highered.thecb_importer.utils import (InstitutionFuzzyMatcher,
        create_or_update)


class Node(object):
    html_parser = HTMLParser.HTMLParser()

    def __init__(self, line):
        self.data = line.strip().replace('<br>', '')

        self.is_empty = False
        self.is_number = False
        self.is_institution = False
        self.is_page_break = False
        self.is_row_header = False

        unescaped_data = self.html_parser.unescape(self.data)

        # Mark nodes we don't care about as empty
        if not self.data or 'BODY>' in self.data or 'HTML>' in self.data:
            self.is_empty = True

        # HR elements signify page breaks
        elif self.data == '<hr>':
            self.is_page_break = True

        # Sometimes multiple numbers appear in the same textbox.
        # We only need the last one since we only care about totals.
        elif re.match(r'^[\d,]+(\s[\d,]+)*$', self.data):
            self.is_number = True
            last_number = self.data.split()[-1].replace(',', '')
            self.data = int(last_number)

        # Institutions are the only non-numeric uppercase lines
        elif unescaped_data.upper() == unescaped_data:
            self.is_institution = True
            self.data = unescaped_data

        elif self.data in ('Total Texas', 'Top 10%',
                           'Enrolled, other Texas public university'):
            self.is_row_header = True

    def __repr__(self):
        return u'<Node: %r>' % self.data


class Parser(object):
    def __init__(self, path):
        self.path = path
        self.data = defaultdict(dict)

        # Parse year from path name
        name = os.path.basename(path).replace('.pdf.html', '')
        self.year = int(name.split('_')[1])

        # Store parser state
        self.cache = []
        self.in_body = False
        self.institution = None
        self.expected_field = None

    def feed(self, line):
        node = Node(line)
        # print node

        # The body begins after the first page break
        if node.is_page_break:
            self.in_body = True
            self.institution = None
            return

        # Skip everything before the body
        if not self.in_body:
            return

        # Return if the node is empty
        if node.is_empty:
            return

        # Expect data after seeing an institution
        if node.is_institution:
            self.institution = node.data

        # If we reach the end of a row and expect data, the last field
        # of the row contains the value for the expected field.
        if node.is_row_header and self.expected_field and self.cache:
            institution_data = self.data[self.institution]
            institution_data[self.expected_field] = self.cache[-1].data
            self.expected_field = None
            self.cache = []

        # Cache numbers until finding an expected value
        elif node.is_number:
            self.cache.append(node)

        # Set expected field from the row header
        if not self.institution:
            return
        if node.data == 'Total Applicants':
            self.expected_field = 'applied'
        elif node.data == 'Total Accepted':
            self.expected_field = 'accepted'
        elif node.data == 'Total Enrolled':
            self.expected_field = 'enrolled'

    def parse(self):
        for line in open(self.path):
            self.feed(line)


def derive_rate(numerator, denominator):
    if denominator == 0 or numerator > denominator:
        return None
    else:
        return round(100.0 * numerator / denominator, 2)


def main(root):
    for path in glob.glob(os.path.join(root, '*.pdf.html')):
        parser = Parser(path)
        parser.parse()
        matcher = InstitutionFuzzyMatcher()
        for institution, data in parser.data.iteritems():
            attrs = dict(institution=institution, year=parser.year, **data)

            # Derive acceptance and enrollment rates
            acceptance_rate = derive_rate(data['accepted'], data['applied'])
            enrollment_rate = derive_rate(data['enrolled'], data['accepted'])

            # Create or update institution admissions for this year
            institution = matcher.match(institution)
            defaults = {
                'year_type': 'fall',
                'number_of_applicants': data['applied'],
                'number_admitted': data['accepted'],
                'number_admitted_who_enrolled': data['enrolled'],
                'percent_of_applicants_admitted': acceptance_rate,
                'percent_of_admitted_who_enrolled': enrollment_rate
            }
            obj, row_count = create_or_update(PublicAdmissions.objects,
                    institution=institution, year=parser.year,
                    defaults=defaults)
            if obj:
                print 'created %s %d admissions...' % (
                    institution.name, parser.year)
            else:
                print 'updated %s %d admissions...' % (
                    institution.name, parser.year)


if __name__ == '__main__':
    main(sys.argv[1])



#! /usr/bin/env python
#
# DEPRECATED This logic is too difficult to maintain. Use the accountability
# search data for graduation rates instead.
#
# Almanac graduation rates are on page 21 of the 2012 THECB Almanac.
# We are only interested in the 2011 rates; the others are loaded from
# the accountability reports.
#
# To preprocess:
#
# $ brew install pdf2json jsonpp
# $ pdf2json -f 21 -l 21 THECB2012almanacweb20120516.pdf
# $ jsonpp THECB2012almanacweb20120516 > almanac_p21.pdf.json
#
from __future__ import absolute_import

import HTMLParser
import json
import re
import sys

from tx_highered.models import Institution, PublicGraduationRates
from tx_highered.thecb_importer.utils import (InstitutionFuzzyMatcher,
        create_or_update)

NAME_RE = re.compile(r"^([^\d\/\%]+)")


class Node(object):
    def __init__(self, text):
        self.__dict__.update(text)
        self.left = text['left']
        self.top = text['top']
        self.data = text['data']

    @property
    def in_bounds(self):
        return self.left < 580 and 300 < self.top < 1420


class Parser(object):
    def __init__(self):
        self._cache = []
        self._rows = []
        self._html_parser = HTMLParser.HTMLParser()

    def _clean_row(self, row):
        # Join all data in the row
        text = ' '.join([node.data for node in row])
        # Preprocess 'n/a' values so all values can be split along '%'
        text = text.replace(' n /a  ', '%n/a%')
        # Extract the institution name so only values remain
        match = NAME_RE.match(text)
        assert match
        name = match.group(0)
        values = text.replace(name, '')
        # Clean name and parse any HTML entities
        cleaned_name = re.sub('\s+', ' ', name).strip('* ')
        cleaned_name = self._html_parser.unescape(cleaned_name)
        # Clean values
        cleaned_values = [v.strip() for v in values.split('%') if v.strip()]
        cleaned_values = [v if v != 'n/a' else None for v in cleaned_values]
        # Expect 13 values: 2000 through 2011 and percent change
        assert len(cleaned_values) == 13
        # The 2011 value is the second to last
        return cleaned_name, cleaned_values[-2]

    def flush(self):
        if self._cache:
            self._rows.append(self._cache)
            self._cache = []

    def feed(self, text):
        node = Node(text)
        if node.in_bounds:
            self._cache.append(node)
        else:
            self.flush()

    def iter_results(self):
        self.flush()
        for row in self._rows:
            yield self._clean_row(row)


def main(path):
    # Parse 2011 6-year graduation rates
    json_text = open(path).read().decode('iso-8859-1')
    page_data = json.loads(json_text)[0]
    parser = Parser()
    for el in page_data['text']:
        parser.feed(el)

    # Match institutions by name and create or update
    matcher = InstitutionFuzzyMatcher()
    for name, bachelor_6yr in parser.iter_results():
        institution = matcher.match(name)
        defaults = dict(bachelor_6yr=bachelor_6yr)
        obj, row_count = create_or_update(PublicGraduationRates.objects,
                institution=institution, year=2011, defaults=defaults)
        if obj:
            print "created %s graduation rates..." % institution.name
        else:
            print "updated %s graduation rates..." % institution.name


if __name__ == '__main__':
    main(sys.argv[1])



#! /usr/bin/env python
from django.conf import settings
try:
    from django.utils.timezone import now
except ImportError:
    from datetime.datetime import now

import os
import requests
from lxml.html import document_fromstring, tostring

from tx_highered.models import Institution


USER_AGENT = 'thedp-scraper/0.1alpha'


def get_wiki_title(name):
    endpoint = "http://en.wikipedia.org/w/api.php"
    params = dict(action="opensearch",
        search=name,
        limit=1,
        namespace=0,
        format="json",)
    r = requests.get(endpoint, params=params)
    try:
        _, results = r.json
        title = results[0]
    except IndexError:
        return None
    return title


def get_wiki_abstract(url):
    r = requests.get(url, headers={'User-Agent': USER_AGENT})
    doc = document_fromstring(r.text)
    root = doc
    try:
        toc = root.get_element_by_id('toc')
    except KeyError:
        return None
    abstract = []
    for elem in toc.getparent().iterchildren():
        if elem == toc:
            break
        if elem.tag == 'p':
            elem.make_links_absolute(url)
            abstract.append(tostring(elem))
    return "\n".join(abstract).strip()


def get_wiki_images(inst):
    # TODO move into the model? has unfulfilled requirements: lxml and requests
    url = inst.wikipedia_url
    r = requests.get(url, headers={'User-Agent': USER_AGENT})
    doc = document_fromstring(r.text)
    images = doc.xpath('//table[@class="infobox vcard"]//a[@class="image"]/img/@src')

    # look for a seal
    seals = [x for x in images if x.lower().find('seal') != -1]
    try:
        seal = seals[0]
    except IndexError:
        seal = None
    if seal and not inst.wikipedia_seal:
        src = "http:" + seal
        # download to settings.MEDIA_ROOT/seals, upload_to="seals"
        dst_path = "seals/%s-seal.png" % inst.slug
        dst_abspath = os.path.join(settings.MEDIA_ROOT, dst_path)
        r = requests.get(src, headers={'User-Agent': USER_AGENT})
        with open(dst_abspath, "wb") as f:
            f.write(r.content)
        inst.wikipedia_seal = dst_path
        inst.save()

    # now look for a logo using the exact same code as above for some reason
    logos = [x for x in images if x not in seals]
    try:
        logo = logos[0]
    except IndexError:
        logo = None
    if logo and not inst.wikipedia_logo:
        src = "http:" + logo
        # download to settings.MEDIA_ROOT/logos, upload_to="logos"
        dst_path = "logos/%s-logo.png" % inst.slug
        dst_abspath = os.path.join(settings.MEDIA_ROOT, dst_path)
        r = requests.get(src, headers={'User-Agent': USER_AGENT})
        with open(dst_abspath, "wb") as f:
            f.write(r.content)
        inst.wikipedia_logo = dst_path
        inst.save()

    return inst


def get_titles():
    qs = queryset.filter(wikipedia_title__isnull=True)
    for inst in qs:
        title = get_wiki_title(inst.name)
        if title:
            inst.wikipedia_title = title
            inst.save()
            print inst.name + " -> " + title


def get_abstracts():
    qs = queryset.filter(wikipedia_title__isnull=False, wikipedia_scraped=None)
    for inst in qs:
        text = get_wiki_abstract(inst.wikipedia_url)
        if text:
            inst.wikipedia_abstract = text
            inst.wikipedia_scraped = now()
            inst.save()
            print inst


def get_images():
    qs = queryset.filter(wikipedia_title__isnull=False)
    for inst in qs:
        print get_wiki_images(inst)


if __name__ == "__main__":
    queryset = Institution.objects.filter(institution_type='uni')



from __future__ import division
from collections import namedtuple
import math

from django.db import models

from ..instachart.models import SimpleChart

from .base import APP_LABEL, Institution

"""
field names are slugified().replace('-', '_') versions of the labels given
in the IPEDS source

"""

__all__ = ['PriceTrends', 'TestScores', 'Admissions',
'Enrollment', 'GraduationRates']

# GENDER_CHOICES = (
#     ('Men', 'Men'),
#     ('Women', 'Women'),
#     ('Total', 'Total'))


# RACE_CHOICES = (
#     ('nra', 'Nonresident alien'),
#     ('white', 'White'),
#     ('black', 'African American'),
#     ('hispanic', 'Hispanic'),
#     ('native', 'Native American'),
#     ('asian', 'Asian/Pacific'),
#     ('unknown', 'Race/ethnicity unknown'))


# IPEDS_RACE_CHOICES = (
#     ('nra', 'Nonresident alien'),
#     ('white', 'White non-Hispanic'),
#     ('black', 'Black non-Hispanic'),
#     ('hispanic', 'Hispanic'),
#     ('native', 'American Indian or Alaska Native'),
#     ('asian', 'Asian or Pacific Islander'),
#     ('unknown', 'Race/ethnicity unknown'))


class InstitutionValueManager(models.Manager):
    def by_year(self, year, **attrs):
        table = self.model._meta.db_table
        institution_table = Institution._meta.db_table

        select = {'year': '%s.year' % table}
        for attr, column in attrs.items():
            select[attr] = '%s.%s' % (table, column)

        return Institution.objects.published().extra(
            tables=[table],
            select=select,
            where=['%s.institution_id = %s.id' % (table, institution_table),
                   '%s.year = %%s' % table],
            params=[year],
        )


class YearBasedInstitutionStatModel(models.Model):
    """ base class """
    YEAR_TYPE_CHOICES = (
        ('academic', 'Academic'),  # 2003-2004
        ('calendar', 'Calendar'),  # 2004
        ('fall', 'Fall'),          # F04
        ('fiscal', 'FY'),          # FY 2010
        ('aug', 'August 31st'))    # August
    year = models.IntegerField(default=1970, verbose_name=u'Year')
    year_type = models.CharField(max_length=10, choices=YEAR_TYPE_CHOICES, null=True)
    institution = models.ForeignKey('Institution', related_name='%(class)s')

    # eehhhh, this doesn't really belong here but whatever
    chart_excluded_fields = ('id', 'institution',)

    class Meta:
        abstract = True
        app_label = APP_LABEL
        ordering = ['year']
        unique_together = ('year', 'institution')

    def __unicode__(self):
        return u"%s" % self.year

    def get_display_year(self):
        if not self.year_type or self.year_type == 'calendar':
            return self.year
        elif self.year_type == 'academic':
            return "%d-%d" % (self.year - 1, self.year)
        elif self.year_type == 'fall':
            # return "F%02d" % (int(self.year) % 100)
            return "Fall %s" % self.year
        elif self.year_type == 'fiscal':
            return "FY %s" % self.year
        elif self.year_type == 'aug':
            return "August 31st, %s" % self.year
        return "%d %s" % (self.year, self.year_type)

    @property
    def display_year(self):
        return self.get_display_year()


class PriceTrends(YearBasedInstitutionStatModel, SimpleChart):
    tuition_fees_in_state = models.IntegerField(null=True,
        verbose_name=u"In-State Tuition & Fees")
    tuition_fees_outof_state = models.IntegerField(null=True,
        verbose_name=u"Out-Of-State Tuition & Fees")
    books_and_supplies = models.IntegerField(null=True,
        verbose_name=u"Books & Supplies")
    room_and_board_on_campus = models.IntegerField(null=True)
    room_and_board_off_campus = models.IntegerField(null=True)
    room_and_board_off_campus_w_family = models.IntegerField(null=True)

    data_source = 'IPEDS'

    @property
    def in_state(self):
        return self.tuition_fees_in_state

    @property
    def out_of_state(self):
        return self.tuition_fees_outof_state

    @property
    def in_state_has_risen(self):
        old = self.a_decade_ago
        return old.in_state < self.in_state

    @property
    def out_of_state_has_risen(self):
        old = self.a_decade_ago
        return old.out_of_state < self.out_of_state

    @property
    def a_decade_ago(self):
        if not hasattr(self, '_a_decade_ago'):
            try:
                self._a_decade_ago = (self.institution.pricetrends
                        .filter(year__lte=self.year - 10)
                        .latest('year'))
            except PriceTrends.DoesNotExist:
                self._a_decade_ago = None
        return self._a_decade_ago

    @property
    def in_state_change(self):
        old = self.a_decade_ago
        change = (self.in_state - old.in_state) / old.in_state
        return round(math.ceil(abs(change * 100)))

    @property
    def out_of_state_change(self):
        old = self.a_decade_ago
        change = (self.out_of_state - old.out_of_state) / old.out_of_state
        return round(math.ceil(abs(change * 100)))

    def __unicode__(self):
        return "Price Trends %s %s" % (self.display_year, self.institution)

    chart_series = (('year', "%d"),
                    ('tuition_fees_in_state', "$%d"),
                    ('tuition_fees_outof_state', "$%d"),
                    ('books_and_supplies', "$%d"))

    chart_head_attrs = (('tuition_fees_in_state', ('data-tablebars=1',)),
                        ('tuition_fees_outof_state', 'data-tablebars=1'),
                        ('books_and_supplies', 'data-tablebars=1'))

    chart_body_attrs = (('tuition_fees_in_state', 'data-value="%d"'),
                        ('tuition_fees_outof_state', 'data-value="%d"'),
                        ('books_and_supplies', 'data-value="%d"'))


#############################################################################
#   Admissions
#############################################################################

class TestScoresManager(models.Manager):
    def latest(self):
        return super(TestScoresManager, self).latest('year')

    def oldest(self):
        return self.order_by('year')[0]


class TestScores(YearBasedInstitutionStatModel):
    sat_verbal_25th_percentile = models.IntegerField(null=True)
    sat_verbal_75th_percentile = models.IntegerField(null=True)
    sat_math_25th_percentile = models.IntegerField(null=True)
    sat_math_75th_percentile = models.IntegerField(null=True)
    sat_writing_25th_percentile = models.IntegerField(null=True)
    sat_writing_75th_percentile = models.IntegerField(null=True)
    sat_submitted_number = models.IntegerField(null=True)
    sat_submitted_percent = models.IntegerField(null=True)
    act_composite_25th_percentile = models.IntegerField(null=True)
    act_composite_75th_percentile = models.IntegerField(null=True)
    act_english_25th_percentile = models.IntegerField(null=True)
    act_english_75th_percentile = models.IntegerField(null=True)
    act_math_25th_percentile = models.IntegerField(null=True)
    act_math_75th_percentile = models.IntegerField(null=True)
    act_writing_25th_percentile = models.IntegerField(null=True)
    act_writing_75th_percentile = models.IntegerField(null=True)
    act_submitted_number = models.IntegerField(null=True)
    act_submitted_percent = models.IntegerField(null=True)

    data_source = 'IPEDS'

    objects = TestScoresManager()

    def __unicode__(self):
        return "Test Scores %s %s" % (self.display_year, self.institution)

    def __json__(self):
        return {
            'year': self.year,
            'sat': {
                'verbal': [self.sat_verbal_25th_percentile, self.sat_verbal_75th_percentile],
                'math': [self.sat_math_25th_percentile, self.sat_math_75th_percentile],
                'writing': [self.sat_writing_25th_percentile, self.sat_writing_75th_percentile],
                'submitted_number': self.sat_submitted_number,
                'submitted_percent': self.sat_submitted_percent,
            },
            'act': {
                'verbal': [self.act_english_25th_percentile, self.act_english_75th_percentile],
                'math': [self.act_math_25th_percentile, self.act_math_75th_percentile],
                'writing': [self.act_writing_25th_percentile, self.act_writing_75th_percentile],
                'composite': [self.act_composite_25th_percentile, self.act_composite_75th_percentile],
                'submitted_number': self.act_submitted_number,
                'submitted_percent': self.act_submitted_percent,
            },
        }

    @property
    def sat_verbal_range(self):
        return "%s - %s" % (self.sat_verbal_25th_percentile,
                self.sat_verbal_75th_percentile)

    @property
    def sat_verbal_range_english(self):
        return self.sat_verbal_range.replace(' - ', ' to ')

    @property
    def sat_math_range(self):
        return "%s - %s" % (self.sat_math_25th_percentile,
                self.sat_math_75th_percentile)

    @property
    def sat_math_range_english(self):
        return self.sat_math_range.replace(' - ', ' to ')

    @property
    def sat_writing_range(self):
        if self.sat_writing_25th_percentile:
            return "%s - %s" % (self.sat_writing_25th_percentile,
                    self.sat_writing_75th_percentile)
        else:
            return ""

    @property
    def sat_writing_range_english(self):
        return self.sat_writing_range.replace(' - ', ' to ')

    @property
    def bar(self):
        if hasattr(self, '_bar'):
            return self._bar
        HorizontalBar = namedtuple('HorizontalBar', ['left', 'width'])
        context = dict()
        min, max = 300, 800 * 1.05  # yeah yeah, i'm overriding max and min
        w = (max - min) / 100
        if self.sat_verbal_25th_percentile:
            context['sat_v'] = HorizontalBar((self.sat_verbal_25th_percentile - min) / w,
                      (self.sat_verbal_75th_percentile - self.sat_verbal_25th_percentile) / w)
        if self.sat_math_25th_percentile:
            context['sat_m'] = HorizontalBar((self.sat_math_25th_percentile - min) / w,
                      (self.sat_math_75th_percentile - self.sat_math_25th_percentile) / w)
        if self.sat_writing_25th_percentile:
            context['sat_w'] = HorizontalBar((self.sat_writing_25th_percentile - min) / w,
                      (self.sat_writing_75th_percentile - self.sat_writing_25th_percentile) / w)

        min, max = 10, 36 * 1.05
        w = (max - min) / 100
        if self.act_english_25th_percentile:
            context['act_e'] = HorizontalBar((self.act_english_25th_percentile - min) / w,
                      (self.act_english_75th_percentile - self.act_english_25th_percentile) / w)
        if self.act_math_25th_percentile:
            context['act_m'] = HorizontalBar((self.act_math_25th_percentile - min) / w,
                      (self.act_math_75th_percentile - self.act_math_25th_percentile) / w)
        if self.act_composite_25th_percentile:
            context['act_c'] = HorizontalBar((self.act_composite_25th_percentile - min) / w,
                      (self.act_composite_75th_percentile - self.act_composite_25th_percentile) / w)
        min, max = 2, 12 * 1.05
        w = (max - min) / 100
        if self.act_writing_25th_percentile:
            context['act_w'] = HorizontalBar((self.act_writing_25th_percentile - min) / w,
                      (self.act_writing_75th_percentile - self.act_writing_25th_percentile) / w)
        self._bar = context
        return context

    @property
    def sat_verbal_width(self):
        return self.sat_verbal_75th_percentile - self.sat_verbal_25th_percentile


class AdmissionsManager(models.Manager):
    def latest(self):
        return super(AdmissionsManager, self).latest('year')

    def oldest(self):
        return self.order_by('year')[0]


class Admissions(YearBasedInstitutionStatModel, SimpleChart):
    number_of_applicants = models.IntegerField(null=True, blank=True)
    number_admitted = models.IntegerField(null=True, blank=True)
    number_admitted_who_enrolled = models.IntegerField(null=True, blank=True)
    percent_of_applicants_admitted = models.DecimalField(max_digits=4, decimal_places=1, null=True, blank=True, verbose_name=u"%admitted")
    percent_of_admitted_who_enrolled = models.DecimalField(max_digits=4, decimal_places=1, null=True, blank=True, verbose_name=u"%admitted who enrolled")
    percent_top10rule = models.DecimalField(max_digits=4, decimal_places=1,
        null=True, verbose_name="First-Time Students in Top 10%")

    data_source = 'IPEDS'

    objects = AdmissionsManager()

    def __unicode__(self):
        return "Admissions Data %s %s" % (self.display_year, self.institution)

    class Meta(YearBasedInstitutionStatModel.Meta):
        get_latest_by = 'year'


class Enrollment(YearBasedInstitutionStatModel, SimpleChart):
    institution_values = InstitutionValueManager()
    objects = models.Manager()

    total = models.IntegerField(null=True)
    fulltime_equivalent = models.IntegerField(null=True,
        verbose_name='Full-time Equivalent')
    fulltime = models.IntegerField(null=True, verbose_name='Full-time')
    parttime = models.IntegerField(null=True, verbose_name='Part-time')
    # TODO better list that works with IPEDS and THECB
    total_percent_white = models.IntegerField(null=True,
        verbose_name='% White')
    total_percent_black = models.IntegerField(null=True,
        verbose_name='% Black')
    total_percent_hispanic = models.IntegerField(null=True,
        verbose_name='% Hispanic')
    total_percent_native = models.IntegerField(null=True,
        verbose_name='% Native Am.')
    total_percent_asian = models.IntegerField(null=True,
        verbose_name='% Asian')
    total_percent_unknown = models.IntegerField(null=True,
        verbose_name='% N/A')

    data_source = 'IPEDS'

    def __unicode__(self):
        return "Enrollment Data %s %s" % (self.display_year, self.institution)

    race_attrs = ['total_percent_%s' % race for race in
            ('white', 'black', 'hispanic', 'native', 'asian', 'unknown')]

    def race_data(self):
        data = []
        for race_attr in self.race_attrs:
            value = getattr(self, race_attr, None)
            if value is not None:
                data.append({
                    'year': self.year,
                    'metric': race_attr,
                    'race': self._meta.get_field(race_attr).verbose_name,
                    'enrollment': self.total,
                    'value': value,
                })

        return data

    chart_series = ('year',
                    'fulltime_equivalent',
                    'fulltime',
                    'parttime',) + tuple(race_attrs)


class GraduationRates(YearBasedInstitutionStatModel, SimpleChart):
    bachelor_4yr = models.IntegerField(null=True,
        verbose_name=u"Bachelor degree within four years")
    bachelor_5yr = models.IntegerField(null=True,
        verbose_name=u"Bachelor degree within five years")
    bachelor_6yr = models.IntegerField(null=True,
        verbose_name=u"Bachelor degree within six years")

    data_source = 'IPEDS'

    def __unicode__(self):
        return "Graduation Rates %s %s" % (self.display_year, self.institution)

    chart_series = ('display_year',
                    ('bachelor_4yr', "%s%%"),
                    ('bachelor_5yr', "%s%%"),
                    ('bachelor_6yr', "%s%%"))



from lxml import html as etree


class DictReader(object):
    fieldnames = None
    mapping = None

    def __init__(self, filehandle, **kwargs):
        doc = etree.parse(filehandle)
        # TODO better header detection
        # TODO know when to stop
        rows = doc.xpath('//tr')

        self.rows = rows

        if 'fieldnames' in kwargs:
            self.fieldnames = kwargs['fieldnames']
        else:
            fieldnames = [x.text_content().strip() for x in rows[0].getchildren()]
            self.fieldnames = fieldnames

        if 'mapping' in kwargs:
            self.mapping = kwargs['mapping']

    def normalize_keys(self, data):
        if self.mapping is None:
            return data
        for key in data.keys():
            if key in self.mapping:
                value = data.pop(key)
                data[self.mapping[key]] = value

    def __iter__(self):
        return self.readrow_as_dict()

    def readrow_as_dict(self):
        for row in self.rows[1:]:
            data = dict(zip(self.fieldnames, [x.text_content().strip() for x in row.getchildren()]))
            self.normalize_keys(data)
            yield data



"""
Attempt to assign as many IPED IDs to Institutions as this script can find

Requires downloading a list of institutions from http://nces.ed.gov/ipeds

"""
import sys

from nltk.metrics import edit_distance as distance

from tx_highered.models import Institution
from . import utils

filename = sys.argv[1]

reader = utils.DictReader(open(filename, 'r'))

qs = Institution.objects.filter(ipeds_id__isnull=True)
total = qs.count()
count = 0

# pass one, exact matches
maybes = []
for row in reader:
    try:
        inst = qs.get(name=row['Name'])
        inst.ipeds_id = row['UnitID']
        inst.save()
        print inst
        count += 1
        continue
    except Institution.DoesNotExist:
        pass
        # print "MISSING:", row['Name']
    maybes.append(row)
    # print row

print "Expected %s Found %s" % (total, count)


def closest(name):
    for m in maybes:
        m['distance'] = distance(Institution.get_unique_name(name), Institution.get_unique_name(m['Name']))
    ordered = sorted(maybes, key=lambda x: x['distance'])
    return ordered[0], ordered[1]['distance'] - ordered[0]['distance']

# pass two, fun stuff
qs = Institution.objects.filter(ipeds_id__isnull=True)
haves = Institution.objects.filter(ipeds_id__isnull=False).values_list('ipeds_id', flat=True)
maybes = [x for x in maybes if int(x['UnitID']) not in haves]
for inst in qs:
    guess, confidence = closest(inst.name)
    if confidence > 0:
        if guess['distance'] == 0:
            inst.ipeds_id = guess['UnitID']
            inst.save()
    print "%d +%d\t" % (guess['distance'], confidence), inst.name, "==", guess['Name']



# -*- coding: UTF-8 -*-
from __future__ import unicode_literals

from optparse import make_option

from django.core.management.base import BaseCommand
from django.db.models import Model, Q

from tx_highered.models import Institution


class Command(BaseCommand):
    help = (
        'Create a report card of our data quality. Run it through '
        'http://setosa.io/csv-fingerprint/ for a quick visualization.'
    )
    option_list = BaseCommand.option_list + (
        make_option('--year',
            dest='year',
            type='int',
            help='Data must be from at least this year'),
    )

    def handle(self, *args, **options):

        def latest_year(model_or_queryset, relation=None):
            """Latest year institution has data, returns year or ''."""
            if relation is not None and isinstance(model_or_queryset, Model):
                queryset = getattr(model_or_queryset, relation).all()
            else:
                queryset = model_or_queryset
            try:
                year = queryset.order_by('-year')[0].year
                if options['year'] is not None and year < options['year']:
                    return ''
                else:
                    return year
            except IndexError:
                return ''

        header = (
            'slug',
            'type',
            'Admissions IPEDS',
            'Admissions THECB',
            'Test Scores',
            'Full-Time Equivalent',
            'Demographics IPEDS',
            'Demographics THECB',
            'Price Trends',
            'Graduation IPEDS',
            'Graduation THECB',
        )
        print ','.join(header)
        institutions = (Institution.objects.published()
            .order_by('institution_type', 'is_private', 'name'))
        for institution in institutions:
            row = [institution.slug, institution.institution_type]
            row.append(latest_year(institution, 'admissions'))
            if institution.is_private:
                row.append('NA')
            else:
                row.append(latest_year(institution, 'publicadmissions'))

            row.append(latest_year(institution, 'testscores'))

            # Enrollment
            #
            # We only care about fulltime equivalent, which is only in IPEDS
            # for now.
            row.append(latest_year(institution.enrollment.filter(
                fulltime_equivalent__isnull=False)))

            # Demographics
            #
            # Assume that every school at least has white or black students
            row.append(latest_year(institution.enrollment.filter(
                Q(total_percent_white__isnull=False) |
                Q(total_percent_black__isnull=False)
            )))
            if institution.is_private:
                row.append('NA')
            else:
                row.append(latest_year(institution.publicenrollment.filter(
                    Q(white_percent__isnull=False) |
                    Q(african_american_percent=False)
                )))

            row.append(latest_year(institution, 'pricetrends'))

            # Bachelor Degree Graduation Rates
            #
            # We currently only care about bachelor's degrees, but will
            # hopefully update the codebase to display associates degrees too.
            row.append(latest_year(institution.graduationrates.filter(
                Q(bachelor_4yr__isnull=False) |
                Q(bachelor_6yr__isnull=False),
            )))
            if institution.is_private:
                row.append('NA')
            else:
                row.append(latest_year(institution.publicgraduationrates.filter(
                    Q(bachelor_4yr__isnull=False) |
                    Q(bachelor_6yr__isnull=False),
                )))

            print ','.join(map(unicode, row))



from __future__ import division

from django.db import models

from .reports import (YearBasedInstitutionStatModel, AdmissionsManager,
        InstitutionValueManager)
from ..instachart.models import SimpleChart


__all__ = ['PublicEnrollment', 'PublicGraduationRates', 'PublicAdmissions']


"""
RACES = [
    ,
    'American Indian/Alaskan Native',
    'Asian',
    'Hispanic',
    'International',
    'Multiracial',
    'Native Hawaiian/Pacific Island',
    'Unknown or Not Reported',
    'White',
]
"""


class PublicEnrollment(YearBasedInstitutionStatModel, SimpleChart):
    institution_values = InstitutionValueManager()
    objects = models.Manager()

    total = models.IntegerField(null=True, help_text='Total full-time students')
    # TODO fulltime_equivalent = models.IntegerField(null=True,
    #    verbose_name='Full-time Equivalent')

    african_american_count = models.IntegerField(null=True)
    asian_count = models.IntegerField(null=True)
    hispanic_count = models.IntegerField(null=True)
    international_count = models.IntegerField(null=True)
    multiracial_count = models.IntegerField(null=True)
    native_american_count = models.IntegerField(null=True)
    pacific_islander_count = models.IntegerField(null=True)
    unknown_count = models.IntegerField(null=True)
    white_count = models.IntegerField(null=True)

    african_american_percent = models.FloatField(null=True,
        verbose_name='% Black')
    asian_percent = models.FloatField(null=True,
        verbose_name='% Asian')
    hispanic_percent = models.FloatField(null=True,
        verbose_name='% Hispanic')
    international_percent = models.FloatField(null=True,
        verbose_name='% International')
    multiracial_percent = models.FloatField(null=True,
        verbose_name='% Multiracial')
    native_american_percent = models.FloatField(null=True,
        verbose_name='% Native Am.')
    pacific_islander_percent = models.FloatField(null=True,
        verbose_name='% Pacific Islander')
    unknown_percent = models.FloatField(null=True,
        verbose_name='% Unknown')
    white_percent = models.FloatField(null=True,
        verbose_name='% White')

    data_source = 'THECB'

    @property
    def total_percent_white(self):
        return self.white_percent

    @property
    def total_percent_black(self):
        return self.african_american_percent

    @property
    def total_percent_hispanic(self):
        return self.hispanic_percent

    @property
    def total_percent_native(self):
        return self.native_american_percent

    @property
    def total_percent_asian(self):
        return self.asian_percent

    @property
    def total_percent_unknown(self):
        return self.unknown_percent

    def race_data(self):
        race_attrs = [f.attname for f in self._meta.fields
                      if '_percent' in f.attname]
        data = []
        for race_attr in race_attrs:
            value = getattr(self, race_attr, None)
            if value is not None:
                data.append({
                    'year': self.year,
                    'metric': race_attr,
                    'race': self._meta.get_field(race_attr).verbose_name.title(),
                    'enrollment': self.total,
                    'value': value,
                })

        return data

    race_attrs = ['total_percent_%s' % race for race in
            ('white', 'black', 'hispanic', 'native', 'asian', 'unknown')]

    chart_series = (('year', 'total') +
                    tuple(race_attrs) +
                    ('international_percent', 'multiracial_percent',
                     'pacific_islander_percent'))


class PublicGraduationRates(YearBasedInstitutionStatModel, SimpleChart):
    associate_3yr = models.DecimalField(null=True,
        max_digits=5, decimal_places=2,
        verbose_name=u"Associate degree within three years",
        help_text=u"Only available for community colleges")
    associate_4yr = models.DecimalField(null=True,
        max_digits=5, decimal_places=2,
        verbose_name=u"Associate degree within four years",
        help_text=u"Only available for community colleges")
    associate_6yr = models.DecimalField(null=True,
        max_digits=5, decimal_places=2,
        verbose_name=u"Associate degree within six years",
        help_text=u"Only available for community colleges")

    bachelor_3yr = models.DecimalField(null=True,
        max_digits=5, decimal_places=2,
        verbose_name=u"Bachelor degree within three years",
        help_text=u"Only available for community colleges")
    bachelor_4yr = models.DecimalField(null=True,
        max_digits=5, decimal_places=2,
        verbose_name=u"Bachelor degree within four years")
    bachelor_5yr = models.DecimalField(null=True,
        max_digits=5, decimal_places=2,
        verbose_name=u"Bachelor degree within five years")
    bachelor_6yr = models.DecimalField(null=True,
        max_digits=5, decimal_places=2,
        verbose_name=u"Bachelor degree within six years")

    data_source = 'THECB'

    def __unicode__(self):
        return "Graduation Rates %s %s" % (self.display_year, self.institution)

    chart_series = ('display_year',
                    ('bachelor_4yr', "%s%%"),
                    ('bachelor_5yr', "%s%%"),
                    ('bachelor_6yr', "%s%%"))


class PublicAdmissions(YearBasedInstitutionStatModel, SimpleChart):
    number_of_applicants = models.IntegerField(null=True, blank=True)
    number_admitted = models.IntegerField(null=True, blank=True)
    number_admitted_who_enrolled = models.IntegerField(null=True, blank=True)
    percent_of_applicants_admitted = models.DecimalField(max_digits=4, decimal_places=1, null=True, blank=True, verbose_name=u"%admitted")
    percent_of_admitted_who_enrolled = models.DecimalField(max_digits=4, decimal_places=1, null=True, blank=True, verbose_name=u"%admitted who enrolled")

    data_source = 'THECB'

    objects = AdmissionsManager()

    class Meta(YearBasedInstitutionStatModel.Meta):
        get_latest_by = 'year'



from setuptools import setup

setup(
    name='tx_highered',
    version='0.3.3',
    description='Django app for Texas higher education data',
    author='Texas Tribune',
    author_email='tech@texastribune.org',
    url='http://github.com/texastribune/tx_highered/',
    license='Apache Software License',
    install_requires=[
        'tx_lege_districts>=0.4.0',
    ],
    packages=['tx_highered'],
    include_package_data=True,  # automatically include things from MANIFEST
    zip_safe=True,
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Apache Software License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Other/Nonlisted Topic'
    ],
)



#! /usr/bin/env python
# -*- coding: UTF-8 -*-
"""
Usage: html_to_csv.py FILE

Convert an excel html file to csv on stdout. Similar to "in2csv", except in2csv
does not support this format.

Arguments:
  FILE  one *.xls.html file from THECB
"""

import os
import sys

from csvkit import CSVKitDictWriter
from docopt import docopt
from lxml import html as etree


def parse_html(path):
    html = etree.parse(path)
    row_iter = iter(html.xpath('//tr'))

    # Throw away the report header in the first row
    first_row = next(row_iter)
    if len(first_row.getchildren()) == 1:
        first_row = next(row_iter)

    # The column names are in the second row
    header = [col.text_content() for col in first_row]

    # Parse dict records from the remaining rows
    data = []
    for row in row_iter:
        values = [c.text.strip().strip('%') if c.text else None for c in row]
        values = [v if v != 'N/A' else None for v in values]
        record = dict(zip(header, values))
        data.append(record)

    return header, data


def html_to_csv(path):
    header, data = parse_html(path)

    # Create stdout writer
    header.insert(2, 'System')
    writer = CSVKitDictWriter(sys.stdout, header)
    writer.writeheader()

    # Parse ID and system before writing each row
    for row in data:
        id_field = row.pop('FICE')
        if not id_field:
            continue

        # Parse out System that may be in parenthesis in the FICE field
        fice_parts = id_field.split(' (')
        row['FICE'] = fice_parts[0]
        if len(fice_parts) > 1:
            row['System'] = fice_parts[1].strip(') ')
        else:
            row['System'] = None

        writer.writerow(row)


if __name__ == '__main__':
    arguments = docopt(__doc__)
    if not os.path.exists(arguments['FILE']):
        exit("File does not exist")
    html_to_csv(arguments['FILE'])



import factory

from . import models


class InstitutionFactory(factory.DjangoModelFactory):
    FACTORY_FOR = models.Institution

    name = factory.Sequence(lambda i: u'Institution {}'.format(i))
    slug = factory.Sequence(lambda i: u'institution-{}'.format(i))


class BaseReportFactory(factory.DjangoModelFactory):
    institution = factory.SubFactory(InstitutionFactory)


class EnrollmentFactory(BaseReportFactory):
    FACTORY_FOR = models.Enrollment


class PublicEnrollmentFactory(BaseReportFactory):
    FACTORY_FOR = models.PublicEnrollment



# -*- coding: UTF-8 -*-
from __future__ import unicode_literals

from django.core.management.base import BaseCommand

from tx_highered.models import Admissions


class Command(BaseCommand):
    help = 'Post-process data higher ed data. Safe to run multiple times.'

    def handle(self, *args, **options):
        # Backfill missing `percent_of_applicants_admitted`
        for x in Admissions.objects.filter(percent_of_applicants_admitted=None, number_of_applicants__gt=0, number_admitted__gt=0):
            x.percent_of_applicants_admitted = \
                x.number_admitted * 1000 / x.number_of_applicants / 10.0
            print x, '% admitted', x.percent_of_applicants_admitted
            x.save()

        # Backfill missing `percent_of_admitted_who_enrolled`
        for x in Admissions.objects.filter(percent_of_admitted_who_enrolled__isnull=True, number_admitted_who_enrolled__gt=0, number_admitted__gt=0):
            x.percent_of_admitted_who_enrolled = \
                x.number_admitted_who_enrolled * 1000 / x.number_admitted / 10.0
            print x, '% enrolled', x.percent_of_admitted_who_enrolled
            x.save()

        # Other things that we could/should be denormalizing would be done here



import os

from django.core.management.base import BaseCommand, CommandError

from tx_highered.scripts.import_customreport import generic as ipeds
from tx_highered.scripts.import_thecb_report import generic as thecb


class Command(BaseCommand):
    args = '(ipeds|thecb) <file file ...>'
    help = "Import Data"

    def handle(self, importer_type, *args, **options):
        # WISHLIST handle verbosity option

        if importer_type == 'ipeds':
            for path in args:
                if os.path.isfile(path):
                    ipeds(path)
        elif importer_type == 'thecb':
            for path in args:
                if os.path.isfile(path):
                    thecb(path)
        else:
            raise CommandError(u'Not a valid importer type: "{}"'
                .format(importer_type))
